/////////////////////////////////////////////////////////////////////
//
// AdjustZTImagePosition.java :
//
//  Window for Adjusting a ZT Image's Position.
//
/////////////////////////////////////////////////////////////////////
//
// $Revision: 10 $
// $Date: 12/29/09 2:16p $
//
/////////////////////////////////////////////////////////////////////

package com.madsci.zoot;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.util.*;
import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

/**
 * Window for Adjusting a ZT Image's Position.
 */

// Jay changed the following from JDialog FixRotDlg to JFrame
// AdjustZTImagePosition to allow this window to be maximized,
// which is good for larger images.

public class AdjustZTImagePosition extends JFrame
        implements MouseWheelListener,
        MouseMotionListener,
        MouseListener
{
	static final String FIXROTGRIDWIDTH  = "FixRotGridWidth";
	static final String FIXROTGRIDHEIGHT = "FixRotGridHeight";

	JPanel contentPane;
	BorderLayout blayContentPane = new BorderLayout();
		JScrollPane scrollPane = new JScrollPane();
			PixPanel picPane = new PixPanel();
		JPanel northPane = new JPanel();
			JPanel settingsPane = new JPanel();
				JLabel labelFootprint = new JLabel(Res.get("FOOTPRINT"));

				// Jay changed "getLastGridWidth()" to 2
				// because that routine does not know
				// the owning frame yet.

				JSpinner spinnerWidth = new
				  JSpinner(new SpinnerNumberModel(2, 1,
				                                  100, 1));

				// Jay changed the following line to make it
				// clearer what the field represents.

				JLabel labelTimes =
				  new JLabel(Res.get("FOOTPRINTY"));

				// Jay changed "getLastGridHeight()" to 2
				// because that routine does not know
				// the owning frame yet.

				JSpinner spinnerHeight = new
				  JSpinner(new SpinnerNumberModel(2, 1,
				                                  100, 1));
				JPanel panelStrut = new JPanel();
				JPanel panelStrut2 = new JPanel();
				JLabel labelOffsets = new JLabel(Res.get("OFFSETS"));
				JLabel labelMag = new JLabel(":");
				JSpinner spinnerMag = new JSpinner(new SpinnerNumberModel(1.0, 1.0, 5.0, 0.1));

				// Jay added the following 5 lines
				// to create a help button.

				JPanel helpButtonStrut = new JPanel();
				JButton helpButton =
				  new JButton(new ImageIcon(
				  com.madsci.zoot.MainFrame.class.getResource(
				                         "images/help.png")));
		JPanel buttonsPane = new JPanel();

			// Jay added the following 16 lines to create
			// buttons that can be used to move the image.

			JButton downButton =
			  new JButton(new ImageIcon(
			       com.madsci.zoot.MainFrame.class.getResource(
		                               "images/downArrow16x16.png")));
			JButton upButton =
			  new JButton(new ImageIcon(
			       com.madsci.zoot.MainFrame.class.getResource(
		                                 "images/upArrow16x16.png")));
			JButton leftButton =
			  new JButton(new ImageIcon(
			       com.madsci.zoot.MainFrame.class.getResource(
		                               "images/leftArrow16x16.png")));
			JButton rightButton =
			  new JButton(new ImageIcon(
			       com.madsci.zoot.MainFrame.class.getResource(
		                              "images/rightArrow16x16.png")));

			// Jay changed the following from "SAVEFILE"
			// to "SAVEFILEANDCLOSEWINDOW".

			JButton jbOk =
			  new JButton(Res.get("SAVEFILEANDCLOSEWINDOW"));

			// Jay added the following 2 lines to create buttons
			// that allow going to the next or prior frame.

			JButton jbNext = new JButton(Res.get("NEXTFRAME"));
			JButton jbPrior = new JButton(Res.get("PRIORFRAME"));
			JButton jbCancel = new JButton(Res.get("CANCEL"));

	ImageIcon imgMag;

	// Jay added the following line to remember if this was invoked
	// to adjust each of the individual frames.

	private boolean adjustIndividualFrames;

	// Jay added the following line to allow remembering the owning Frame.

	private MainFrame owningFrame;
	private ZTImage zti = null;
	private int nFrameNum = 0;
	private BufferedImage bi;

	private BufferedImage biGridOutline = null;

	//
	// Bounding rectangle of the grid outline image
	//
	Rectangle rectPic = new Rectangle();
	Rectangle rectBackground = new Rectangle();

	//
	// This takes care of repainting whenever the spinner values change
	//
	SpinnerChangeListener spinnerChangeListener = new SpinnerChangeListener();

	//
	// The point in the picture where the user clicked
	//
	Point clickPoint = null;

	class PixPanel extends JPanel
	{
		public static final int SQUARE_W = 30;

		public void paint(Graphics g)
		{
			super.paint(g);

			Graphics2D g2 = (Graphics2D)g;

			//
			// Scale the coords
			//
			AffineTransform origTrans = g2.getTransform();
			AffineTransform at = (AffineTransform)origTrans.clone();
			at.translate(getOrigXOffset(), getOrigYOffset());
			at.scale(getScaleFactor(), getScaleFactor());
			g2.setTransform(at);

			//
			// Paint the underlying grid
			//
			paintGrid(g2);

			//
			// Display our picture
			//
			g2.drawImage(bi,
			             rectPic.x, rectPic.y,
			             rectPic.width, rectPic.height,
			             null);

			g2.setTransform(origTrans);
		}

		/**
		 * Paints a grid under the picture to help with alignment.
		 * @param g2
		 */
		public void paintGrid(Graphics2D g2)
		{
			//
			// Get the size of the image's footprint
			//
			// Jay changed the original 2 lines to the following
			// 16 lines so that the footprint would be
			// automatically reversed for the SW and NE views.
			// The file name usually includes the path,
			// which would include a backslash in MS Windows
			// and a forward slash in Linux.

			int nH;
			int nW;
			String imageFileName = zti.sFileName.toUpperCase();

			if (imageFileName.equals("SW")
			    || imageFileName.equals("NE")
			    || imageFileName.endsWith("\\SW")
			    || imageFileName.endsWith("\\NE")
			    || imageFileName.endsWith("/SW")
			    || imageFileName.endsWith("/NE"))
			{ nH = ((Integer)spinnerWidth.getValue()).intValue();
			  nW = ((Integer)spinnerHeight.getValue()).intValue();
			}
			else
			{ nW = ((Integer)spinnerWidth.getValue()).intValue();
			  nH = ((Integer)spinnerHeight.getValue()).intValue();
			}

			int nGridW = biGridOutline.getWidth();
			int nGridH = biGridOutline.getHeight();

			int nBoundingBoxW = (nW + nH) * nGridW / 4;
			int nBoundingBoxH = (nW + nH) * nGridH / 4;

			rectBackground.x = nBoundingBoxW;
			rectBackground.y = nBoundingBoxH;

			AffineTransform at = g2.getTransform();
			AffineTransform at2 = (AffineTransform)at.clone();

			//
			// Center the whole grid to the origin.
			//
			at2.translate(-((nBoundingBoxW / 2) - ((nH - 2) * nGridW / 4)),
			              (nBoundingBoxH / 2) - nGridH);
			g2.setTransform(at2);

			for (int y = 0; y < nH; y+=2)
			{
				for (int x = 0; x < nW; x+=2)
				{
					int posX = x * nGridW / 4;
					int posY = -x * nGridH / 4;

					int offsetX = -y * nGridW / 4;
					int offsetY = -y * nGridH / 4;

					//
					// Draw the grid outline image using a transparent background
					//
					g2.drawImage(biGridOutline,
					             posX + offsetX, posY + offsetY,
					             nGridW, nGridH,
					             null, null);
				}
			}

			g2.setTransform(at);
		}
	} // class PixPanel

	// Jay changed the following constructor from allowing the adjustment
	// of a specific frame to remembering whether each individual frame
	// should be adjusted. He also changed from Frame to MainFrame.
	// He also added the throws, after removing the Exception handling
	// from the constructor, so that the calling class can see it
	// and handle it more gracefully.  He also changed FixRotDlg
	// to AdjustZTImagePosition to reflect the change from JDialog
	// to JFrame.

	public AdjustZTImagePosition(ZTImage img, boolean doIndividualFrames,
	                             MainFrame owner)
	       throws Exception
	{
		// Jay changed the following line when this class was changed
		// from JDialog to JFrame.

		super();

		// Jay added the following line to remember if this
		// window is to adjust each individual frame.

		adjustIndividualFrames = doIndividualFrames;

		// Jay added the following line to remember the owning Frame.

		owningFrame = owner;
		zti = img;

		// Jay changed the following line to always start at frame 0.

		nFrameNum = 0;

		enableEvents(AWTEvent.WINDOW_EVENT_MASK);
		jbInit();
	}

	//Component initialization
	private void jbInit() throws Exception
	{
		// Jay added the following 8 lines to change the text
		// of the title and "Ok" and "Cancel" buttons, if this window
		// is to adjust the position of each individual frame
		// in the image.

		if (adjustIndividualFrames == true)
		{ Object args[] = { zti.sFileName, new Integer(nFrameNum + 1),
		                    new Integer(zti.getNumFrames()) };
		  this.setTitle(Res.format("FIXFRAMESTITLE", args));
		  jbOk.setText(Res.get("SAVEFILE"));
		  jbCancel.setText(Res.get("CLOSEWINDOW"));
		}
		else
		{ Object args[] = { zti.sFileName };
		  this.setTitle(Res.format("FIXEM", args));
		}

		// Jay added the following 3 lines to set the window's icon.

		this.setIconImage
		  ((new ImageIcon(com.madsci.zoot.MainFrame.class.getResource
		                  ("images/zoot16x16.png"))).getImage());

		// Jay added the following 2 lines to set width and height
		// spinner values to what they were the last time.
		// This has to be done as part of initialization rather than
		// during object creation since the owning frame was not set
		// until initialization.

		spinnerWidth.setValue(new Integer(getLastGridWidth()));
		spinnerHeight.setValue(new Integer(getLastGridHeight()));

		//
		// The magnifier pic for the spinner legend
		//
		imgMag = new ImageIcon(com.madsci.zoot.MainFrame.class.getResource(
			"images/mag16x16.png"));
		labelMag.setIcon(imgMag);

		//
		// Stupid *struts*
		//
		panelStrut.setMinimumSize(new Dimension(150, 20));
		panelStrut2.setMinimumSize(new Dimension(150, 20));

		contentPane = (JPanel)this.getContentPane();
		contentPane.setLayout(blayContentPane);

		// Jay increased the width from 500 to 600 if this was started
		// for positioning each individual frame and to 530 if this
		// was started for positioning the image as a whole.
		// These sizes accommodate the added buttons.

		if (adjustIndividualFrames == true)
		{ contentPane.setPreferredSize(new Dimension(600, 500));
		}
		else
		{ contentPane.setPreferredSize(new Dimension(530, 500));
		}

		contentPane.add(scrollPane, BorderLayout.CENTER);

		scrollPane.setViewportView(picPane);

		//
		// Footprint combos and current offsets
		//
		FlowLayout fl = (FlowLayout)settingsPane.getLayout();
		fl.setAlignment(FlowLayout.LEADING);
		settingsPane.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
		settingsPane.add(labelFootprint);
		settingsPane.add(spinnerWidth);
		settingsPane.add(labelTimes);
		settingsPane.add(spinnerHeight);
		settingsPane.add(panelStrut);
		settingsPane.add(labelMag);
		settingsPane.add(spinnerMag);
		settingsPane.add(panelStrut2);
		settingsPane.add(labelOffsets);

		// Jay added the following 7 lines to add a help button
		// to the settings section.

		helpButtonStrut.setMinimumSize(new Dimension(150, 20));
		settingsPane.add(helpButtonStrut);
		helpButton.setMargin(new Insets(0, 0, 0, 0));
		helpButton.setToolTipText(Res.get("MENUHELPFORWINDOW"));
		helpButton.setMnemonic(KeyEvent.VK_H);
		helpButton.addActionListener(helpActionListener);
		settingsPane.add(helpButton);
		spinnerWidth.setPreferredSize(new Dimension(50, 20));
		spinnerHeight.setPreferredSize(new Dimension(50, 20));
		spinnerMag.setPreferredSize(new Dimension(50, 20));

		northPane.setLayout(new BoxLayout(northPane, BoxLayout.Y_AXIS));
		northPane.add(settingsPane);

		contentPane.add(northPane, BorderLayout.NORTH);

		//
		// Ok/Cancel buttons
		//
		jbOk.setMnemonic(KeyEvent.VK_S);
		jbOk.addActionListener(okActionListener);
		jbCancel.setMnemonic(KeyEvent.VK_C);
		jbCancel.addActionListener(cancelActionListener);

		// Jay added the following 16 lines to show
		// the buttons that can be used to move the image.

		downButton.setToolTipText(Res.get("TIPDOWNBUTTON"));
		downButton.setMnemonic(KeyEvent.VK_D);
		downButton.addActionListener(downActionListener);
		buttonsPane.add(downButton);
		upButton.setToolTipText(Res.get("TIPUPBUTTON"));
		upButton.setMnemonic(KeyEvent.VK_U);
		upButton.addActionListener(upActionListener);
		buttonsPane.add(upButton);
		leftButton.setToolTipText(Res.get("TIPLEFTBUTTON"));
		leftButton.setMnemonic(KeyEvent.VK_L);
		leftButton.addActionListener(leftActionListener);
		buttonsPane.add(leftButton);
		rightButton.setToolTipText(Res.get("TIPRIGHTBUTTON"));
		rightButton.setMnemonic(KeyEvent.VK_R);
		rightButton.addActionListener(rightActionListener);
		buttonsPane.add(rightButton);

		// Jay added the following 12 lines to show the buttons
		// that can be used to go to the next and prior frames,
		// if this window is to adjust the position of each
		// individual frame in the image.

		if (adjustIndividualFrames == true)
	        { if (nFrameNum >= zti.getNumFrames() - 1)
	          { jbNext.setEnabled(false);
	          }

	          if (nFrameNum < 1)
	          { jbPrior.setEnabled(false);
		  }

		  jbNext.addActionListener(nextActionListener);
		  buttonsPane.add(jbNext);
		  jbPrior.addActionListener(priorActionListener);
		  buttonsPane.add(jbPrior);
		}

		buttonsPane.add(jbOk);
		buttonsPane.add(jbCancel);

		contentPane.add(buttonsPane, BorderLayout.SOUTH);

		picPane.addMouseWheelListener(this);
		picPane.addMouseMotionListener(this);
		picPane.addMouseListener(this);

		//
		// Make Ok and Cancel respond to Enter and Escape, respectively
		//
		InputMap im = this.getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), "okAction");
		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "cancelAction");

		// Jay added the following 5 lines to map the h and ctrl-h
		// keys to the help action.

		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_H, 0),
		       "helpAction");
		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_H,
		                              ActionEvent.CTRL_MASK),
		       "helpAction");

		// Jay added the following 7 lines to map
		// the d/u/l/r keys to down/up/left/right actions.

		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_D, 0),
		       "downAction");
		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_U, 0), "upAction");
		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_L, 0),
		       "leftAction");
		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_R, 0),
		       "rightAction");
		ActionMap am = this.getRootPane().getActionMap();
		am.put("okAction", okActionListener);
		am.put("cancelAction", cancelActionListener);

		// Jay added the following line to map the help action
		// to its method.

		am.put("helpAction", helpActionListener);

		// Jay added the following 4 lines to map
		// the down/up/left/right actions to their methods.

		am.put("downAction", downActionListener);
		am.put("upAction", upActionListener);
		am.put("leftAction", leftActionListener);
		am.put("rightAction", rightActionListener);

		bi  = zti.toBufferedImage(nFrameNum);

		rectPic.x = -zti.getOffsets(nFrameNum).x;
		rectPic.y = -zti.getOffsets(nFrameNum).y;
		rectPic.setSize(bi.getWidth(), bi.getHeight());

		// Jay changed the following to get the grid image name
		// to use from the owning frame.

		biGridOutline =
		  ImageIO.read(com.madsci.zoot.MainFrame.class.getResource(
		                             owningFrame.getGridImageName()));

		spinnerWidth.addChangeListener(this.spinnerChangeListener);
		spinnerHeight.addChangeListener(this.spinnerChangeListener);
		spinnerMag.addChangeListener(this.spinnerChangeListener);
	}

	public void paint(Graphics g)
	{
		Object args[] = { new Integer(-rectPic.x),
		                  new Integer(-rectPic.y) };
		labelOffsets.setText(Res.format("OFFSETSARGS", args));
		super.paint(g);
	}

	private float getScaleFactor() { return ((Double)spinnerMag.getValue()).floatValue(); }
	private float setScaleFactor(float f)
	{
		f += getScaleFactor();
		f = Math.min(5.0F, Math.max(1.0F, f));
		spinnerMag.setValue(new Double(f));
		return f;
	}

	/**
	 * Recalculate the minimum size to display the grid and set the scrollpane's viewport.
	 */
	void recalcViewport()
	{
		Rectangle r = rectBackground.union(rectPic);
		picPane.setPreferredSize(new Dimension((int)(r.width*getScaleFactor()),
		                                       (int)(r.height*getScaleFactor())));
		picPane.revalidate();
		scrollPane.setViewportView(picPane);
	}

	/**
	 * Converts a point in client coords to our coords.
	 *
	 * @param x
	 * @param y
	 * @return the point in unscaled, untranslated coordinates.
	 */
	Point convertToInternalCoords(int x, int y)
	{
		Point p = new Point(x, y);
		p.translate(-getOrigXOffset(), -getOrigYOffset());
		p.x /= getScaleFactor();
		p.y /= getScaleFactor();
		return p;
	}

	int getOrigXOffset() { return picPane.getWidth()/2;	}
	int getOrigYOffset() { return picPane.getHeight()/2; }

	// Jay changed "((MainFrame)getParent())" to "owningFrame"
	// in the following 4 routines.

	int getLastGridWidth()
	{ return owningFrame.prefs.getInt(FIXROTGRIDWIDTH,  2);
	}

	int getLastGridHeight()
	{ return owningFrame.prefs.getInt(FIXROTGRIDHEIGHT, 2);
	}

	void setLastGridWidth(int w)
	{ owningFrame.prefs.putInt(FIXROTGRIDWIDTH,  w);
	}

	void setLastGridHeight(int h)
	{ owningFrame.prefs.putInt(FIXROTGRIDHEIGHT, h);
	}

	/**
	 * Invoked when the mouse wheel is rotated.
	 * @see MouseWheelEvent
	 */
	public void mouseWheelMoved(MouseWheelEvent e)
	{
		setScaleFactor((float)(e.getWheelRotation()) / 10.0F);
	}

	/**
	 * Invoked when a mouse button is pressed on a component and then
	 * dragged.  <code>MOUSE_DRAGGED</code> events will continue to be
	 * delivered to the component where the drag originated until the
	 * mouse button is released (regardless of whether the mouse position
	 * is within the bounds of the component).
	 * <p>
	 * Due to platform-dependent Drag&Drop implementations,
	 * <code>MOUSE_DRAGGED</code> events may not be delivered during a native
	 * Drag&Drop operation.
	 */
	public void mouseDragged(MouseEvent e)
	{
		//
		// Had the user actually clicked on the picture before dragging?
		//
		if (clickPoint == null)
			return;

		//
		// Make sure the user doesn scroll the image "out" of the window
		//
		Point newPos = convertToInternalCoords(Math.max(10, e.getX()), Math.max(20, e.getY()));

		//
		// Also make sure the image isn't dragged into negative offsets
		//
		// Since some ZT objects have negative offsets, the following
		// 2 lines were modified by Jay to allow negative offsets
		// down to -127, which is a value that can be saved
		// in a signed byte, although it looks like offsets
		// have a greater range than signed bytes.  Since the offsets
		// are relative values, that reverses the sign, so 127
		// is used below instead of -127.

		newPos.x = Math.min(newPos.x - clickPoint.x, 127);
		newPos.y = Math.min(newPos.y - clickPoint.y, 127);

		rectPic.setLocation(newPos.x, newPos.y);
		repaint();
	}

	public void mouseMoved(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {}

	/**
	 * Invoked when a mouse button has been pressed on a component.
	 */
	public void mousePressed(MouseEvent e)
	{
		//
		// See if the picture was clicked
		//
		Point newClick = convertToInternalCoords(e.getX(), e.getY());
		if (!rectPic.contains(newClick))
		{
			clickPoint = null;
			return;
		}

		//
		// Note the point in the picture where the click was made
		//
		clickPoint = new Point(newClick.x - rectPic.x, newClick.y - rectPic.y);
	}

	/**
	 * Invoked when a mouse button has been released on a component.
	 */
	public void mouseReleased(MouseEvent e)
	{
		clickPoint = null;
		recalcViewport();
	}

	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}

	// Jay added the following method to show the help box
	// via the help button.

	public void help_actionPerformed(ActionEvent e)
	{ AdjustZTImagePosition_HelpBox dlg =
	    new AdjustZTImagePosition_HelpBox(this);
	  Dimension dlgSize = dlg.getPreferredSize();
	  Dimension frmSize = getSize();
	  Point loc = getLocation();

	  dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x,
	                  (frmSize.height - dlgSize.height) / 2 + loc.y);
	  dlg.pack();
	  dlg.show();
	}

	// Jay added the following method to move the image down
	// via the down button.

	void down_actionPerformed(ActionEvent e)
	{ rectPic.setLocation(rectPic.x, rectPic.y + 1);
	  repaint();
	}

	// Jay added the following method to move the image up
	// via the up button.

	void up_actionPerformed(ActionEvent e)
	{ rectPic.setLocation(rectPic.x, rectPic.y - 1);
	  repaint();
	}

	// Jay added the following method to move the image left
	// via the left button.

	void left_actionPerformed(ActionEvent e)
	{ rectPic.setLocation(rectPic.x - 1, rectPic.y);
	  repaint();
	}

	// Jay added the following method to move the image right
	// via the right button.

	void right_actionPerformed(ActionEvent e)
	{ rectPic.setLocation(rectPic.x + 1, rectPic.y);
	  repaint();
	}

	void jbOk_actionPerformed(ActionEvent e)
	{	// Since this action can potentially use a lot of resources,
		// Jay added the following 2 lines to free up any used
		// resources.

		System.runFinalization();
		System.gc();
		int nRes = JOptionPane.showConfirmDialog(this,
		                                         Res.get("SAVECONFIRM"),
		                                         Res.get("SAVECONFIRMTITLE"),
		                                         JOptionPane.YES_NO_OPTION,
		                                         JOptionPane.QUESTION_MESSAGE);
		if (nRes == JOptionPane.NO_OPTION)
			return;

		// Jay added the following 4 lines to set the offsets either
		// for all frames or for a specific frame.

		if (adjustIndividualFrames == false)
		{ zti.setAllOffsets(new Point(-rectPic.x, -rectPic.y));
		}
		else
		{ zti.setOffsets(nFrameNum, new Point(-rectPic.x, -rectPic.y));
		}

		try
		{
			zti.save(zti.sRootDir, zti.sFileName);

			// Jay added the following line to refresh the tree
			// in the main frame, since this image has changed.

			owningFrame.refresh();

			// Jay added the following 2 lines to only dispose
			// this window if this window was started to adjust
			// an image as a whole.

			if (adjustIndividualFrames == false)
			{ bi = null;
			  this.dispose();
			}
		}
		catch (Exception e1)
		{	// Jay added the following line to provide more
			// information of the exception.

			e1.printStackTrace();
			Object args[] = { zti.sRootDir + File.separator + zti.sFileName, e1.toString() };
			JOptionPane.showMessageDialog(this, e1.toString(),
			                              Res.format("ERR_FILESAVE", args), JOptionPane.ERROR_MESSAGE);
		}

		// Jay added the following 8 lines so that a popup message
		// occurs for unexpected virtual machine errors,
		// such as out of memory.

		catch (VirtualMachineError e1)
		{ e1.printStackTrace();
		  Object args[] = { e1.toString() };
		  JOptionPane.showMessageDialog(this,
		                           Res.format("ERR_UNEXPECTED", args),
		                                Res.get("ERR_ZOOT"),
		                                JOptionPane.ERROR_MESSAGE);
		}
	}

	//
	// Grrr....
	//
	private class ZTAction implements Action
	{
		public Object getValue(String key) { return null; }
		public void putValue(String key, Object value) {}
		public void setEnabled(boolean b) {}
		public boolean isEnabled() { return true; }
		public void addPropertyChangeListener(PropertyChangeListener listener) {}
		public void removePropertyChangeListener(PropertyChangeListener listener) {}
		public void actionPerformed(ActionEvent e) {}
	}

	// Jay added the following method to listen for events
	// associated with the help button.

	ZTAction helpActionListener = new ZTAction()
	{ public void actionPerformed(ActionEvent e)
	  { help_actionPerformed(e);
	  }
	};

	// Jay added the following 4 methods to listen
	// for the events associated with the image movement buttons.

	ZTAction downActionListener = new ZTAction()
	{ public void actionPerformed(ActionEvent e)
	  { down_actionPerformed(e);
	  }
	};

	ZTAction upActionListener = new ZTAction()
	{ public void actionPerformed(ActionEvent e)
	  { up_actionPerformed(e);
	  }
	};

	ZTAction leftActionListener = new ZTAction()
	{ public void actionPerformed(ActionEvent e)
	  { left_actionPerformed(e);
	  }
	};

	ZTAction rightActionListener = new ZTAction()
	{ public void actionPerformed(ActionEvent e)
	  { right_actionPerformed(e);
	  }
	};

	ZTAction okActionListener = new ZTAction()
	{
		public void actionPerformed(ActionEvent e) { jbOk_actionPerformed(e); }
	};

	// Jay added the following method to go to the next frame of an image,
	// if there is one.

	void jbNext_actionPerformed(ActionEvent e)
	{ if (nFrameNum < zti.getNumFrames() - 1)
	  { try
	    { System.runFinalization();
	      System.gc();
	      zti.setOffsets(nFrameNum, new Point(-rectPic.x, -rectPic.y));
	      ++nFrameNum;
	      Object args[] = { zti.sFileName, new Integer(nFrameNum + 1),
	                        new Integer(zti.getNumFrames()) };
	      this.setTitle(Res.format("FIXFRAMESTITLE", args));

	      if (nFrameNum >= zti.getNumFrames() - 1)
	      { jbNext.setEnabled(false);
	      }

	      jbPrior.setEnabled(true);
	      bi  = zti.toBufferedImage(nFrameNum);
	      rectPic.x = -zti.getOffsets(nFrameNum).x;
	      rectPic.y = -zti.getOffsets(nFrameNum).y;
	      rectPic.setSize(bi.getWidth(), bi.getHeight());
	      recalcViewport();
	      repaint();
	    }
	    catch (Exception e1)
	    { e1.printStackTrace();
	      Object args[] = { e1.toString() };
	      JOptionPane.showMessageDialog(this,
	                                   Res.format("ERR_UNEXPECTED", args),
	                                    Res.get("ERR_ZOOT"),
	                                    JOptionPane.ERROR_MESSAGE);
	    }
	    catch (VirtualMachineError e1)
	    { e1.printStackTrace();
	      Object args[] = { e1.toString() };
	      JOptionPane.showMessageDialog(this,
	                                   Res.format("ERR_UNEXPECTED", args),
	                                    Res.get("ERR_ZOOT"),
	                                    JOptionPane.ERROR_MESSAGE);
	    }
	  }
	}

	// Jay added the following method to listen for events associated
	// with the "Next frame" button.

	ZTAction nextActionListener = new ZTAction()
	{ public void actionPerformed(ActionEvent e)
	  { jbNext_actionPerformed(e);
	  }
	};

	// Jay added the following method to go to the prior frame
	// of an image, if there is one.

	void jbPrior_actionPerformed(ActionEvent e)
	{ if (nFrameNum > 0)
	  { try
	    { System.runFinalization();
	      System.gc();
	      zti.setOffsets(nFrameNum, new Point(-rectPic.x, -rectPic.y));
	      --nFrameNum;
	      Object args[] = { zti.sFileName, new Integer(nFrameNum + 1),
	                        new Integer(zti.getNumFrames()) };
	      this.setTitle(Res.format("FIXFRAMESTITLE", args));

	      if (nFrameNum < 1)
	      { jbPrior.setEnabled(false);
	      }

	      jbNext.setEnabled(true);
	      bi  = zti.toBufferedImage(nFrameNum);
	      rectPic.x = -zti.getOffsets(nFrameNum).x;
	      rectPic.y = -zti.getOffsets(nFrameNum).y;
	      rectPic.setSize(bi.getWidth(), bi.getHeight());
	      recalcViewport();
	      repaint();
	    }
	    catch (Exception e1)
	    { e1.printStackTrace();
	      Object args[] = { e1.toString() };
	      JOptionPane.showMessageDialog(this,
	                                   Res.format("ERR_UNEXPECTED", args),
	                                    Res.get("ERR_ZOOT"),
	                                    JOptionPane.ERROR_MESSAGE);
	    }
	    catch (VirtualMachineError e1)
	    { e1.printStackTrace();
	      Object args[] = { e1.toString() };
	      JOptionPane.showMessageDialog(this,
	                                   Res.format("ERR_UNEXPECTED", args),
	                                    Res.get("ERR_ZOOT"),
	                                    JOptionPane.ERROR_MESSAGE);
	    }
	  }
	}

	// Jay added the following method to listen for events associated
	// with the "Prior frame" button.

	ZTAction priorActionListener = new ZTAction()
	{ public void actionPerformed(ActionEvent e)
	  { jbPrior_actionPerformed(e);
	  }
	};

	void jbCancel_actionPerformed(ActionEvent e)
	{
		// Jay added the following line to tell Java sooner
		// that the image is no longer used.

		bi = null;

		// Jay added the following 23 lines to refresh the tree
		// in the main frame, if this was started to adjust
		// the position of each frame in the image, in case offsets
		// were changed without saving them to the file.  Since Java
		// uses the same object for both classes, the image
		// in the main frame needs to be reread to get what
		// the offsets were at the time of the last save.

		if (adjustIndividualFrames == true)
		{ System.runFinalization();
		  System.gc();

		  try
		  { owningFrame.refresh();
		  }
		  catch (Exception e1)
		  { e1.printStackTrace();
		    Object args[] = { e1.toString() };
		    JOptionPane.showMessageDialog(this,
		                           Res.format("ERR_UNEXPECTED", args),
		                                  Res.get("ERR_ZOOT"),
		                                  JOptionPane.ERROR_MESSAGE);
		  }
		  catch (VirtualMachineError e1)
		  { e1.printStackTrace();
		    Object args[] = { e1.toString() };
		    JOptionPane.showMessageDialog(this,
		                           Res.format("ERR_UNEXPECTED", args),
		                                  Res.get("ERR_ZOOT"),
		                                  JOptionPane.ERROR_MESSAGE);
		  }
		}

		this.dispose();
	}

	ZTAction cancelActionListener = new ZTAction()
	{
		public void actionPerformed(ActionEvent e) { jbCancel_actionPerformed(e); }
	};

	class SpinnerChangeListener implements ChangeListener
	{
		public void stateChanged(ChangeEvent e)
		{
			if (e.getSource().equals(spinnerWidth))
				setLastGridWidth(((Integer)spinnerWidth.getValue()).intValue());
			else if (e.getSource().equals(spinnerHeight))
				setLastGridHeight(((Integer)spinnerHeight.getValue()).intValue());

			recalcViewport();
			repaint();
		}
	}
}

/********************************************************************
 *
 * $Log: /Zoot/src/com/madsci/zoot/AdjustZTImagePosition.java $
 *
 * 10   12/29/09 2:16p Jay
 * Add help button, add moving of image by buttons and keys, allow adjusting
 * individual frames or adjusting all frames based on first, reverse grid
 * direction for SW and NE views, show unexpected errors, allow negative
 * offsets down to -127, have Java free unused resources sooner, have the main
 * window reread the tree to get any changes, change this class from JDialog
 * to JFrame, and get the grid image name from the owning frame.
 *
 * 9     8/11/03 3:41p Fpoulin
 * Added inline help.
 *
 * 8     8/05/03 12:23p Fpoulin
 *
 * 7     7/16/03 6:48p Fpoulin
 * Use Java Preferences to provide some persistence.
 *
 * 6     7/11/03 1:41a Fpoulin
 * Optimized imports
 *
 * 5     7/08/03 4:50a Fpoulin
 * Performance improvements.
 *
********************************************************************/
