/////////////////////////////////////////////////////////////////////
//
// EditZTImage.java :
//
//  Window for Editing a ZT Image.
//
/////////////////////////////////////////////////////////////////////
//
// $Revision: 8 $
// $Date: 12/28/09 12:07a $
//
/////////////////////////////////////////////////////////////////////

package com.madsci.zoot;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.beans.PropertyChangeListener;
import java.io.*;
import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.table.*;
import javax.swing.tree.*;

/**
 * Window for Editing a ZT Image.
 */

// Jay changed the following from JDialog EditImgDlg to JFrame EditZTImage
// to allow this window to be maximized, which is good for larger images.

public class EditZTImage extends JFrame
{
	public final String CMD_MOVEUP = "MoveUp";
	public final String CMD_MOVEDN = "MoveDn";
	public final String CMD_ADDFRAME = "AddFrame";
	public final String CMD_DELFRAME = "DelFrame";
	public final String CMD_SAVEFRAME = "SaveFrame";

	JPanel contentPane;
	BorderLayout blayContentPane = new BorderLayout();
		JSplitPane mainPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
			PicPanel picPane = new PicPanel();
			JPanel bottomPane = new JPanel();
				JPanel frameControls = new JPanel();
					JButton jbMoveUp = new JButton();
					JButton jbMoveDn = new JButton();
					JButton jbAddFrame = new JButton();
					JButton jbDelFrame = new JButton();
					JButton jbSaveFrame = new JButton();

					// Jay added the following 5 lines
					// to create a help button.

					JPanel helpButtonStrut = new JPanel();
					JButton helpButton =
					  new JButton(new ImageIcon(
					    com.madsci.zoot.MainFrame.class
					    .getResource("images/help.png")));
				JScrollPane scrollPane = new JScrollPane();
					ZTFrameTable tblFrames = new ZTFrameTable();
		JPanel settingsPane = new JPanel();
			JLabel labelFramerate = new JLabel(Res.get("FRAMERATE"));
			JSpinner spinnerFramerate = new JSpinner(new SpinnerNumberModel(0, 0, 65536, 1));
			JLabel labelNumFrames = new JLabel(Res.get("NUMFRAMES"));
			JLabel labelMag = new JLabel(":");
			JSpinner spinnerMag = new JSpinner();
		JPanel buttonsPane = new JPanel();

			// Jay changed the following from "SAVEFILE"
			// to "SAVEFILEANDCLOSEWINDOW".

			JButton jbOk =
			  new JButton(Res.get("SAVEFILEANDCLOSEWINDOW"));
			JButton jbCancel = new JButton(Res.get("CANCEL"));

	ImageIcon imgMag;

	// Jay added the following line to allow remembering the owning Frame.

	private MainFrame owningFrame;

	private ZTImage zti = null;

//	private Point ptZtiOffsets = null;

	private ImageVector vecFrames = new ImageVector();

	//
	// This takes care of changing the framerate whenever the spinner values change
	//
	FramerateSpinnerChangeListener framerateSpinnerChangeListener = new FramerateSpinnerChangeListener();

	// Jay added the following 2 lines for the object that will listen
	// for any selection changes in the frames table.

	FramesTableSelectionListener framesTableSelectionListener =
	  new FramesTableSelectionListener();

	// Jay changed the following from Frame to MainFrame.  He also added
	// the throws, after removing the Exception handling
	// from the constructor, so that the calling class can see it
	// and handle it more gracefully.  He also changed EditImgDlg
	// to EditZTImage to reflect the change from JDialog to JFrame.

	public EditZTImage(ZTImage img, MainFrame owner) throws Exception
	{
		// Jay changed the following line when this class was changed
		// from JDialog to JFrame.

		super();

		// Jay added the following line to remember the owning Frame.

		owningFrame = owner;

		zti = img;

//		ptZtiOffsets = zti.getOffsets(0);

		enableEvents(AWTEvent.WINDOW_EVENT_MASK);
		doInit();
	}

	//Component initialization
	private void doInit() throws Exception
	{
		Object args[] = { zti.sFileName };
		this.setTitle(Res.format("EDITEM", args));

		// Jay added the following 3 lines to set the window's icon.

		this.setIconImage
		  ((new ImageIcon(com.madsci.zoot.MainFrame.class.getResource
		                  ("images/zoot16x16.png"))).getImage());

		//
		// The magnifier pic for the spinner legend
		//
		imgMag = new ImageIcon(com.madsci.zoot.MainFrame.class.getResource(
			"images/mag16x16.png"));
		labelMag.setIcon(imgMag);

		//
		// Load up the frame images
		//
		for (int i = 0; i < zti.getNumFrames(); i++)
			vecFrames.add(zti.toBufferedImage(i));

		contentPane = (JPanel)this.getContentPane();
		contentPane.setLayout(blayContentPane);
		contentPane.setPreferredSize(new Dimension(500, 500));

		contentPane.add(mainPane, BorderLayout.CENTER);

		picPane.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
		picPane.setPreferredSize(new Dimension(vecFrames.getImg(0).getWidth() + 10,
		                                       vecFrames.getImg(0).getHeight() + 10));

		jbMoveUp.setText(Res.get("BTNMOVEUP"));
		jbMoveUp.setToolTipText(Res.get("TIPBTNMOVEUP"));
		jbMoveUp.setActionCommand(CMD_MOVEUP);
		jbMoveUp.addActionListener(moveActionListener);
		jbMoveDn.setText(Res.get("BTNMOVEDN"));
		jbMoveDn.setToolTipText(Res.get("TIPBTNMOVEDN"));
		jbMoveDn.setActionCommand(CMD_MOVEDN);
		jbMoveDn.addActionListener(moveActionListener);

		jbAddFrame.setText(Res.get("BTNADDFRAME"));
		jbAddFrame.setToolTipText(Res.get("TIPBTNADDFRAME"));
		jbAddFrame.setActionCommand(CMD_ADDFRAME);
		jbAddFrame.addActionListener(diskOpActionListener);
		jbDelFrame.setText(Res.get("BTNDELFRAME"));
		jbDelFrame.setToolTipText(Res.get("TIPBTNDELFRAME"));
		jbDelFrame.setActionCommand(CMD_DELFRAME);
		jbDelFrame.addActionListener(diskOpActionListener);
		jbSaveFrame.setText(Res.get("BTNSAVEFRAME"));
		jbSaveFrame.setToolTipText(Res.get("TIPBTNSAVEFRAME"));
		jbSaveFrame.setActionCommand(CMD_SAVEFRAME);
		jbSaveFrame.addActionListener(diskOpActionListener);

		// Jay added the following 5 lines to add a help button
		// to the frame controls section.

		helpButtonStrut.setMinimumSize(new Dimension(150, 20));
		helpButton.setMargin(new Insets(0, 0, 0, 0));
		helpButton.setToolTipText(Res.get("MENUHELPFORWINDOW"));
		helpButton.setMnemonic(KeyEvent.VK_H);
		helpButton.addActionListener(helpActionListener);

		frameControls.add(jbMoveUp);
		frameControls.add(jbMoveDn);
		frameControls.add(Box.createHorizontalStrut(10));
		frameControls.add(jbAddFrame);
		frameControls.add(jbDelFrame);
		frameControls.add(jbSaveFrame);

		// Jay added the following 2 lines to add the help button.

		frameControls.add(helpButtonStrut);
		frameControls.add(helpButton);
		frameControls.setBorder(BorderFactory.createMatteBorder(2, 2, 0, 2, SystemColor.controlHighlight));

		bottomPane.setLayout(new BorderLayout());
		bottomPane.add(frameControls, BorderLayout.NORTH);
		bottomPane.add(scrollPane, BorderLayout.CENTER);

		mainPane.setTopComponent(picPane);
		mainPane.setBottomComponent(bottomPane);

		updateTableModel(new ZTFrameTableModel(vecFrames));

		scrollPane.setViewportView(tblFrames);

		//
		// Framerate and Mag spinners
		//
		spinnerFramerate.setMaximumSize(new Dimension(75, 20));
		spinnerMag.setMaximumSize(new Dimension(50, 20));
		settingsPane.setLayout(new BoxLayout(settingsPane, BoxLayout.X_AXIS));
		settingsPane.setBorder(BorderFactory.createEmptyBorder(5, 0, 5, 0));
		settingsPane.add(Box.createHorizontalStrut(5));
		settingsPane.add(labelFramerate);
		settingsPane.add(Box.createRigidArea(new Dimension(5, 0)));
		settingsPane.add(spinnerFramerate);
		settingsPane.add(Box.createRigidArea(new Dimension(20, 0)));
		settingsPane.add(labelMag);
		settingsPane.add(Box.createRigidArea(new Dimension(5, 0)));
		settingsPane.add(spinnerMag);
		settingsPane.add(Box.createGlue());
		settingsPane.add(labelNumFrames);
		settingsPane.add(Box.createRigidArea(new Dimension(5, 0)));

		contentPane.add(settingsPane, BorderLayout.NORTH);

		//
		// Ok/Cancel buttons
		//
		jbOk.setMnemonic(KeyEvent.VK_S);
		jbOk.addActionListener(okActionListener);
		jbCancel.setMnemonic(KeyEvent.VK_C);
		jbCancel.addActionListener(cancelActionListener);
		buttonsPane.add(jbOk);
		buttonsPane.add(jbCancel);

		contentPane.add(buttonsPane, BorderLayout.SOUTH);

		//
		// Make Ok and Cancel respond to Enter and Escape, respectively
		//
		InputMap im = this.getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), "okAction");
		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "cancelAction");

		// Jay added the following 5 lines to map the h and ctrl-h
		// keys to the help action.

		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_H, 0),
		       "helpAction");
		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_H,
		                              ActionEvent.CTRL_MASK),
		       "helpAction");
		ActionMap am = this.getRootPane().getActionMap();
		am.put("okAction", okActionListener);
		am.put("cancelAction", cancelActionListener);

		// Jay added the following line to map the help action
		// to its method.

		am.put("helpAction", helpActionListener);
		spinnerFramerate.addChangeListener(this.framerateSpinnerChangeListener);
		spinnerMag.setModel(picPane.msm);

		// Jay added the following 4 lines to listen for any selection
		// changes in the frames table.

		tblFrames.getSelectionModel()
		.addListSelectionListener(this.framesTableSelectionListener);
		tblFrames.getColumnModel().getSelectionModel()
		.addListSelectionListener(this.framesTableSelectionListener);

		//
		// Animate the picture
		//
		spinnerFramerate.setValue(new Integer(zti.getFrameRate()));
		picPane.setImage(vecFrames, zti.getFrameRate());
		mainPane.resetToPreferredSizes();
	}

	public void paint(Graphics g)
	{
		super.paint(g);

		// Jay changed the rest of this routine to indicate which
		// which frame is selected, if any.

		setFrameNumber();
	}

	// Jay added the following method to allow other classes to select
	// a specific frame.

	public void selectFrame(int frameNumber)
	{ if (frameNumber >= 0 && frameNumber < vecFrames.size())
	  { ZTFrameTableModel tm = (ZTFrameTableModel)(tblFrames.getModel());
	    int row = tm.getRowOfIndex(frameNumber);
	    int col = tm.getColOfIndex(frameNumber);
	    tblFrames.addRowSelectionInterval(row, row);
	    tblFrames.addColumnSelectionInterval(col, col);
	    tblFrames.requestFocusInWindow();
	    setFrameNumber();
	    Rectangle rectToShow = tblFrames.getCellRect(row, col, false);

	    // If this is not the first row, also show at least part
	    // of the previous row to make it easier to select a frame
	    // from there.

	    if (row > 0)
	    { rectToShow.y -= 10;
	      rectToShow.height += 10;
	    }

	    // If this is not the last row, also show at least part
	    // of the next row to make it easier to select a frame
	    // from there.

	    if (row < tblFrames.getRowCount() - 1)
	    { rectToShow.height += 10;
	    }

	    tblFrames.scrollRectToVisible(rectToShow);
	  }
	}

	// Jay added the following method to indicate which frame
	// is selected, if any.

	private void setFrameNumber()
	{ int nRow = tblFrames.getSelectedRow();
	  int nCol = tblFrames.getSelectedColumn();

	  if (nRow == -1 || nCol == -1)
	  { Object args[] = { new Integer(vecFrames.size()) };
	    labelNumFrames.setText(Res.format("NUMFRAMESARGS", args));
	    return;
	  }

	  ZTFrameTableModel tm = (ZTFrameTableModel)tblFrames.getModel();
	  int nIdx = tm.getIndexOf(nRow, nCol);

	  if (nIdx > vecFrames.size() - 1)
	  { Object args[] = { new Integer(vecFrames.size()) };
	    labelNumFrames.setText(Res.format("NUMFRAMESARGS", args));
	    return;
	  }

	  Object args[] =
	    { new Integer(nIdx + 1), new Integer(vecFrames.size()) };
	  labelNumFrames.setText(Res.format("FRAMENUMARGS", args));
	}

	private float getScaleFactor() { return ((Number)spinnerMag.getValue()).floatValue(); }
	private float setScaleFactor(float f)
	{
		f += getScaleFactor();
		f = Math.min(5.0F, Math.max(1.0F, f));
		spinnerMag.setValue(new Double(f));
		picPane.setPreferredSize(new Dimension((int)(vecFrames.getImg(0).getWidth() * getScaleFactor()) + 10,
		                                       (int)(vecFrames.getImg(0).getHeight() * getScaleFactor()) + 10));
		mainPane.resetToPreferredSizes();
		return f;
	}

	/**
	 * Swaps two contiguous images in the grid.
	 *
	 * @param nRow row of the image to swap
	 * @param nCol column of the image to swap
	 * @param bPrevious if true, swap the image with the previous image. Otherwise swap witht the following image.
	 * @return the new grid position of the image.
	 */
	public Point swapImages(int nRow, int nCol, boolean bPrevious)
	{
		ZTFrameTableModel tm = (ZTFrameTableModel)tblFrames.getModel();
		int nIdx = tm.getIndexOf(nRow, nCol);

		// Jay added the following 3 lines to check if the selected
		// frame is within the range of the image's frames,
		// since it is possible to select an empty frame.

		if (nIdx > vecFrames.size() - 1)
		{ return null;
		}

		if (nIdx == 0 && bPrevious)
			return null;
		if (nIdx >= vecFrames.size() - 1 && !bPrevious)
			return null;

		int nIdxToSwap = bPrevious ? nIdx - 1 : nIdx + 1;

		// Jay changed the following 2 lines from RenderedImage
		// to DisplayImage, which includes offsets.

		DisplayImage ri1 = vecFrames.getImg(nIdx);
		DisplayImage ri2 = vecFrames.getImg(nIdxToSwap);
		vecFrames.set(nIdx, ri2);
		vecFrames.set(nIdxToSwap, ri1);

		updateTableModel(new ZTFrameTableModel(vecFrames));

		//
		// Select the new cell
		//
		int x = tm.getColOfIndex(nIdxToSwap);
		int y = tm.getRowOfIndex(nIdxToSwap);

		tblFrames.clearSelection();
		tblFrames.addColumnSelectionInterval(x, x);
		tblFrames.addRowSelectionInterval(y, y);

		// Jay added the following 3 lines to make sure the newly
		// selected cell is visible and to update the displayed
		// frame number.

		tblFrames.scrollRectToVisible(tblFrames.getCellRect(y, x,
		                                                    false));
		setFrameNumber();
		return new Point(x, y);
	}

	/**
	 * Invoked when the mouse wheel is rotated.
	 * @see java.awt.event.MouseWheelEvent
	 */
	public void mouseWheelMoved(MouseWheelEvent e)
	{
		setScaleFactor((float)(e.getWheelRotation()) / 10.0F);
	}

	// Jay added the following method to show the help box
	// via the help button.

	public void help_actionPerformed(ActionEvent e)
	{ EditZTImage_HelpBox dlg = new EditZTImage_HelpBox(this);
	  Dimension dlgSize = dlg.getPreferredSize();
	  Dimension frmSize = getSize();
	  Point loc = getLocation();

	  dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x,
	                  (frmSize.height - dlgSize.height) / 2 + loc.y);
	  dlg.pack();
	  dlg.show();
	}

	void jbOk_actionPerformed(ActionEvent e)
	{
		// Since this action can potentially use a lot of resources,
		// Jay added the following 2 lines to free up any used
		// resources.

		System.runFinalization();
		System.gc();

		int nRes = JOptionPane.showConfirmDialog(this,
		                                         Res.get("SAVECONFIRM"),
		                                         Res.get("SAVECONFIRMTITLE"),
		                                         JOptionPane.YES_NO_OPTION,
		                                         JOptionPane.QUESTION_MESSAGE);
		if (nRes == JOptionPane.NO_OPTION)
			return;

		try
		{
			//
			// Set the new framerate
			//
			zti.setFrameRate(((Integer)spinnerFramerate.getValue()).intValue());

			//
			// Set the new images
			//
			zti.clearFrames();
			for (int i = 0; i < vecFrames.size(); i++)
			{
			  // Jay added the following 2 lines to also get
			  // the offsets for the image.

			  DisplayImage imageFrame = vecFrames.getImg(i);
			  Point imageOffsets = imageFrame.getOffsets();

			  // Jay changed the following line to also pass
			  // offsets to the frame being created.

			  zti.addFrameFrom((BufferedImage)imageFrame,
			                   (short)imageOffsets.x,
			                   (short)imageOffsets.y);
			}

			//
			// Re-set this 'cuz we just zapped the original value...
			//

//			zti.setOffsets(ptZtiOffsets);

			//
			// Write the file out
			//
			zti.save(zti.sRootDir, zti.sFileName);
			picPane.setImage(null, 0);

			// Jay added the following line to refresh
			// the tree in the main menu, since this image
			// has changed.

			owningFrame.refresh();
			this.dispose();

//			TreePath tp = ((ZTTree)(((MainFrame)getParent()).jTreeFiles)).getSelectionPath();
//			((ZTTree)(((MainFrame)getParent()).jTreeFiles)).refresh();
//			((ZTTree)(((MainFrame)getParent()).jTreeFiles)).setExpandsSelectedPaths(true);
//			((ZTTree)(((MainFrame)getParent()).jTreeFiles)).setSelectionPath(tp);
//			((ZTTree)(((MainFrame)getParent()).jTreeFiles)).scrollPathToVisible(tp);
		}
		catch (Exception e1)
		{	// Jay added the following line to provide more
			// information of the exception.

			e1.printStackTrace();
			Object args[] = { zti.sRootDir + File.separator + zti.sFileName, e1.toString() };
			JOptionPane.showMessageDialog(this, Res.format("ERR_FILESAVE", args),
			                              Res.get("ERR_ZOOT"), JOptionPane.ERROR_MESSAGE);
		}

		// Jay added the following 8 lines so that a popup message
		// occurs for unexpected virtual machine errors,
		// such as out of memory.

		catch (VirtualMachineError e1)
		{ e1.printStackTrace();
		  Object args[] = { e1.toString() };
		  JOptionPane.showMessageDialog(this,
		                           Res.format("ERR_UNEXPECTED", args),
		                                Res.get("ERR_ZOOT"),
		                                JOptionPane.ERROR_MESSAGE);
		}
	}

	public void updateTableModel(ZTFrameTableModel tm)
	{
		tblFrames.setModel(tm);
		Dimension dim = vecFrames.getMaxCellSize();

		//
		// Pain: resize the columns and rows to show largest frame in its entirety
		//
		int nW = (int)dim.getWidth() + 10;
		int nH = (int)dim.getHeight() + 10;

		tblFrames.setRowHeight(nH);

		if (tblFrames.getColumnCount() == 0)
			return;

		if (nW > tblFrames.getColumnModel().getColumn(0).getWidth())
		{
			TableColumn column = null;
			for (int i = 0; i < tblFrames.getColumnCount(); i++)
			{
				column = tblFrames.getColumnModel().getColumn(i);
				column.setPreferredWidth(nW);
			}
		}
	}

	//
	// Grrr....
	//
	private class ZTAction implements Action
	{
		EditZTImage adaptee;

		public ZTAction(EditZTImage jd) { adaptee = jd; }

		public Object getValue(String key) { return null; }
		public void putValue(String key, Object value) {}
		public void setEnabled(boolean b) {}
		public boolean isEnabled() { return true; }
		public void addPropertyChangeListener(PropertyChangeListener listener) {}
		public void removePropertyChangeListener(PropertyChangeListener listener) {}
		public void actionPerformed(ActionEvent e) {}
	}

	// Jay added the following method to listen for events
	// associated with the help button.

	ZTAction helpActionListener = new ZTAction(this)
	{ public void actionPerformed(ActionEvent e)
	  { help_actionPerformed(e);
	  }
	};

	ZTAction okActionListener = new ZTAction(this)
	{
		public void actionPerformed(ActionEvent e) { jbOk_actionPerformed(e); }
	};

	void jbCancel_actionPerformed(ActionEvent e) { picPane.setImage(null, 0); this.dispose(); }

	ZTAction cancelActionListener = new ZTAction(this)
	{
		public void actionPerformed(ActionEvent e) { jbCancel_actionPerformed(e); }
	};

	void jbMove_actionPerformed(boolean bMoveUp)
	{
		// Since this action can potentially use a lot of resources,
		// Jay added the following 2 lines to free up any used
		// resources.

		System.runFinalization();
		System.gc();
		int nCol = tblFrames.getSelectedColumn();
		int nRow = tblFrames.getSelectedRow();

		if (nCol == -1 || nRow == -1)
			return;

		swapImages(nRow, nCol, bMoveUp);

		//
		// Update the real-time animation
		//
		picPane.setImage(vecFrames, ((Integer)spinnerFramerate.getValue()).intValue());
	}

	ZTAction moveActionListener = new ZTAction(this)
	{
		public void actionPerformed(ActionEvent e)
		{ // Jay wrapped this method in a try...catch so that
		  // a popup message occurs for unexpected exceptions
		  // or virtual machine errors, such as out of memory.

		  try
		  {
			if (e.getActionCommand().compareTo(adaptee.CMD_MOVEUP) == 0)
				jbMove_actionPerformed(true);
			else if (e.getActionCommand().compareTo(adaptee.CMD_MOVEDN) == 0)
				jbMove_actionPerformed(false);
		  }
		  catch (Exception e1)
		  { e1.printStackTrace();
		    Object args[] = { e1.toString() };
		    JOptionPane.showMessageDialog(adaptee,
		                           Res.format("ERR_UNEXPECTED", args),
		                                  Res.get("ERR_ZOOT"),
		                                  JOptionPane.ERROR_MESSAGE);
		  }
		  catch (VirtualMachineError e1)
		  { e1.printStackTrace();
		    Object args[] = { e1.toString() };
		    JOptionPane.showMessageDialog(adaptee,
		                           Res.format("ERR_UNEXPECTED", args),
		                                  Res.get("ERR_ZOOT"),
		                                  JOptionPane.ERROR_MESSAGE);
		  }
		}
	};

	void jbAddFrame_actionPerformed(ActionEvent e)
	{
		// Since this action can potentially use a lot of resources,
		// Jay added the following 2 lines to free up any used
		// resources.

		System.runFinalization();
		System.gc();

		// Jay changed "((MainFrame)(getParent()))" to "owningFrame".

		String sTopDir = owningFrame.getLastVisitedDir();
		sTopDir += File.separator;

		JFileChooser jfc = new JFileChooser();
		jfc.setMultiSelectionEnabled(true);
		jfc.setDialogTitle(Res.get("DLGIMPORTIMAGE"));
		jfc.setCurrentDirectory(new File(sTopDir));
		jfc.addChoosableFileFilter(new PNGFileFilter());
		jfc.setAcceptAllFileFilterUsed(false);

		int nRet = jfc.showOpenDialog(this);

		if (nRet != JFileChooser.APPROVE_OPTION)
			return;

		File f[] = jfc.getSelectedFiles();

		if (f == null || f.length == 0)
			return;

		// Jay changed "((MainFrame)(getParent()))" to "owningFrame".

		owningFrame.setLastVisitedDir(f[0].getParent());

		for (int i = 0; i < f.length; i++)
		{
			String sFile = f[i].getPath();

			if (!f[i].exists())
				return;

			try
			{
				BufferedImage image = ImageIO.read(f[i]);

				ZTImage ztimage = new ZTImage(1, 0);
				ztimage.setTransparentColorARGB(image.getRGB(0,0));

				// Jay added the following 9 lines to also
				// pass offset values.  If this is the first
				// frame of the image, the initial offset
				// values are half of the image's width
				// and height, which works the best for icons.
				// If this is not the first frame
				// of the frame, then the offsets
				// from the first frame are used.

				if (vecFrames.size() == 0)
				{ ztimage.addFrameFrom(image,
				              (short)(image.getWidth() / 2),
				              (short)(image.getHeight() / 2));
				}
				else
			  	{ Point firstOffsets =
				         ((DisplayImage)(vecFrames.getImg(0)))
				         .getOffsets();

				  // Jay changed the following line to also
				  // pass offset values.

				  ztimage.addFrameFrom(image,
				                     (short)(firstOffsets.x),
				                     (short)(firstOffsets.y));
				}

				// Jay changed the following line
				// from BufferedImage to DisplayImage,
				// which includes offsets.

				DisplayImage bi = ztimage.toBufferedImage(0);
				vecFrames.add(bi);

				//
				// Update the table's model
				//
				updateTableModel(new ZTFrameTableModel(vecFrames));

				// Jay added the following line to update
				// the displayed frame count.

				setFrameNumber();

				//
				// Update the real-time animation
				//
				picPane.setImage(vecFrames, ((Integer)spinnerFramerate.getValue()).intValue());
				mainPane.resetToPreferredSizes();
			}
			catch (Exception e1)
			{	// Jay added the following line to provide more
				// information of the exception.

				e1.printStackTrace();
				Object args[] = { sFile, e1.toString() };
				JOptionPane.showMessageDialog(this, Res.format("ERR_FILEOPEN", args),
											  Res.get("ERR_ZOOT"), JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	void jbDelFrame_actionPerformed(ActionEvent e)
	{
		// Since this action can potentially use a lot of resources,
		// Jay added the following 2 lines to free up any used
		// resources.

		System.runFinalization();
		System.gc();
		int nRow = tblFrames.getSelectedRow();
		int nCol = tblFrames.getSelectedColumn();

		if (nRow == -1 || nCol == -1)
			return;

		ZTFrameTableModel tm = (ZTFrameTableModel)tblFrames.getModel();
		int nIdx = tm.getIndexOf(nRow, nCol);
		if (nIdx > vecFrames.size() - 1)
			return;

		//
		// Remove the frame from the array
		//
		vecFrames.removeElementAt(nIdx);

		//
		// Update the table's model
		//
		updateTableModel(new ZTFrameTableModel(vecFrames));

		//
		// Update the real-time animation
		//
		picPane.setImage(vecFrames, ((Integer)spinnerFramerate.getValue()).intValue());
		mainPane.resetToPreferredSizes();

		//
		// Select the previous image (if there are any left!)
		//
		if (tm.getImageCount() > 0)
		{
			int nIdxToSelect = Math.max(0, Math.min(tm.getImageCount()-1, nIdx-1));
			int row = tm.getRowOfIndex(nIdxToSelect);
			int col = tm.getColOfIndex(nIdxToSelect);
			tblFrames.addRowSelectionInterval(row, row);
			tblFrames.addColumnSelectionInterval(col, col);
			tblFrames.requestFocusInWindow();

			// Jay added the following 3 lines to make sure
			// the newly selected image is visible.

			tblFrames
			.scrollRectToVisible(tblFrames.getCellRect(row, col,
			                                           false));
		}

		// Jay added the following line to update the displayed
		// frame count.

		setFrameNumber();
	}

	/**
	 * Save the selected ZT frame into an individual PNG image.
	 */
	void jbSaveFrame_actionPerformed(ActionEvent e)
	{
		// Since this action can potentially use a lot of resources,
		// Jay added the following 2 lines to free up any used
		// resources.

		System.runFinalization();
		System.gc();

		if (zti == null)
			return;

		BufferedImage bi = getSelectedImage();

		if (bi == null)
			return;

		//
		// Bring up a File Save dialog
		//
		// Jay changed "((MainFrame)getParent())" to "owningFrame".

		String sFile = owningFrame.selectPNGToSave();
		if (sFile == null)
			return;

		//
		// Make sure the file has a .PNG extension
		//
		if (!zti.isSupportedType(sFile))
			sFile += ".png";

		//
		// If file exists, ask user to confirm overwrite
		//
		File f = new File(sFile);
		if (f.exists())
		{
			Object args[] = { sFile };
			int nRes = JOptionPane.showConfirmDialog(this,
			                                         Res.format("OVERWRITECONFIRM", args),
													 Res.get("OVERWRITECONFIRMTITLE"),
													 JOptionPane.YES_NO_OPTION,
													 JOptionPane.QUESTION_MESSAGE);
			if (nRes == JOptionPane.NO_OPTION)
				return;
		}

		try
		{
			ImageIO.write(bi, "png", f);
		}
		catch (IOException e1)
		{	// Jay added the following line to provide more
			// information of the exception.

			e1.printStackTrace();
			Object args[] = { sFile, e1.toString() };
			JOptionPane.showMessageDialog(this, Res.format("ERR_FILESAVE", args),
			                              Res.get("ERR_ZOOT"), JOptionPane.ERROR_MESSAGE);
		}
	}

	ZTAction diskOpActionListener = new ZTAction(this)
	{
		public void actionPerformed(ActionEvent e)
		{ // Jay wrapped this method in a try...catch so that
		  // a popup message occurs for unexpected exceptions
		  // or virtual machine errors, such as out of memory.

		  try
		  {
			if (e.getActionCommand().compareTo(adaptee.CMD_ADDFRAME) == 0)
				jbAddFrame_actionPerformed(e);
			else if (e.getActionCommand().compareTo(adaptee.CMD_DELFRAME) == 0)
				jbDelFrame_actionPerformed(e);
			else if (e.getActionCommand().compareTo(adaptee.CMD_SAVEFRAME) == 0)
				jbSaveFrame_actionPerformed(e);
		  }
		  catch (Exception e1)
		  { e1.printStackTrace();
		    Object args[] = { e1.toString() };
		    JOptionPane.showMessageDialog(adaptee,
		                           Res.format("ERR_UNEXPECTED", args),
		                                  Res.get("ERR_ZOOT"),
		                                  JOptionPane.ERROR_MESSAGE);
		  }
		  catch (VirtualMachineError e1)
		  { e1.printStackTrace();
		    Object args[] = { e1.toString() };
		    JOptionPane.showMessageDialog(adaptee,
		                           Res.format("ERR_UNEXPECTED", args),
		                                  Res.get("ERR_ZOOT"),
		                                  JOptionPane.ERROR_MESSAGE);
		  }
		}
	};

	public BufferedImage getSelectedImage()
	{
		int nRow = tblFrames.getSelectedRow();
		int nCol = tblFrames.getSelectedColumn();

		if (nRow == -1 || nCol == -1)
			return null;

		ZTFrameTableModel tm = (ZTFrameTableModel)tblFrames.getModel();
		int nIdx = tm.getIndexOf(nRow, nCol);
		if (nIdx > vecFrames.size() - 1)
			return null;
		return vecFrames.getImg(nIdx);
	}

	private class FramerateSpinnerChangeListener implements ChangeListener
	{
		public void stateChanged(ChangeEvent e)
		{
			picPane.setFramerate(((Integer)spinnerFramerate.getValue()).intValue());
		}
	}


	// Jay added the following class to change the selected frame number
	// on a selection change in the frames table.

	private class FramesTableSelectionListener
	  implements ListSelectionListener
	{ public void valueChanged(ListSelectionEvent e)
	  { setFrameNumber();
	  }
	}
}

/********************************************************************
 *
 * $Log: /Zoot/src/com/madsci/zoot/EditZTImage.java $
 *
 * 8    12/28/09 12:07a Jay
 * Add help button, show unexpected errors, add a check so that empty frames
 * will not be attempted to be moved, remember any previous offsets of all
 * frames, have Java free unused resources sooner, have the main window reread
 * the tree to get any changes, have any selected frame shown as much
 * as possible, update the displayed frame count when needed, including any
 * selected frame number, and change this class from JDialog to JFrame.
 *
 * 7     8/11/03 4:53p Fpoulin
 * Allow multiple frame adds in one operation, and streamlined navigation
 * of frame deletion.
 *
 * 6     8/05/03 12:23p Fpoulin
 *
 * 5     8/04/03 8:13p Fpoulin
 * Preserve rotation offsets when saving image.
 *
 * 4     7/11/03 1:44a Fpoulin
 * Added "new image" feature
 *
 * 3     7/09/03 7:26p Fpoulin
 * Added frame import capabilities.
 *
 * 2     7/09/03 4:40p Fpoulin
 * Added frame reordering and saving capabilities.
 *
 * 1     7/08/03 4:51a Fpoulin
 * Image frame edition window.
 *
********************************************************************/

