/////////////////////////////////////////////////////////////////////
//
// MainFrame.java :
//
//  Main Zoot window.
//
/////////////////////////////////////////////////////////////////////
//
// $Revision: 14 $
// $Date: 12/27/09 11:57p $
//
/////////////////////////////////////////////////////////////////////

package com.madsci.zoot;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;

// Jay added the following line to allow enumerating all expanded
// paths in the main tree after a refresh.

import java.util.*;
import java.util.prefs.Preferences;
import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.tree.*;


/**
 * Main Zoot window.
 */
public class MainFrame
	extends JFrame
    implements ChangeListener
{
	//
	// CardLayout card names
	//
	public static final String CARDTXT = "CARDTXT";
	public static final String CARDIMG = "CARDIMG";

	//
	// Preferences object and constants
	//
	public Preferences prefs = Preferences.userNodeForPackage(MainFrame.class);
	public static final String LASTDIR = "LastDir";
	public static final String LASTPROJ = "LastProj";

	JPanel contentPane;

	//
	// Menu Items
	//
	JMenuBar jMenuBarMain = new JMenuBar();
		JMenu jMenuFile = new JMenu();
			JMenuItem jMenuFileOpenProj = new JMenuItem();
			JMenuItem jMenuFileRefresh = new JMenuItem();
			JMenuItem jMenuFileExit = new JMenuItem();
		JMenu jMenuImage = new JMenu();
			JMenuItem jMenuImageNewImg = new JMenuItem();
			JMenuItem jMenuImageEditImg = new JMenuItem();

			// Jay added the following line to allow starting
			// the window for adjusting the position
			// of each frame in an image.

			JMenuItem jMenuImageFixRot = new JMenuItem();
			JMenuItem jMenuImageAdjustFrames = new JMenuItem();
			JMenuItem jMenuImageSave = new JMenuItem();
		JMenu jMenuHelp = new JMenu();

			// Jay added the following line to allow getting help
			// for the window via the Help menu.

			JMenuItem jMenuHelpForWindow = new JMenuItem();
			JMenuItem jMenuHelpAbout = new JMenuItem();

	//
	// Menu command strings
	//
	static final String MENUFILEEXIT = "Exit";
	static final String MENUFILEREFRESH = "Refresh";
	static final String MENUFILEOPENPROJ = "OpenProj";
	static final String MENUIMAGENEWIMG = "NewImg";
	static final String MENUIMAGEEDITIMG = "EditImg";
	static final String MENUIMAGEFIXROT = "FixRot";

	// Jay added the following line to allow starting the window
	// for adjusting the position of each frame in an image.

	static final String MENUIMAGEADJUSTFRAMES = "AdjustFrames";
	static final String MENUIMAGESAVE = "SaveImage";

	//
	// Toolbar
	//
	JToolBar jToolBar = new JToolBar();
		JButton jBtnOpenProject = new JButton();
		JButton jBtnRefresh = new JButton();
		JButton jBtnNewImg = new JButton();
		JButton jBtnEditImg = new JButton();
		JButton jBtnFixRot = new JButton();

		// Jay added the following line to allow starting the window
		// for adjusting the position of each frame in an image.

		JButton jBtnAdjustFrames = new JButton();
		JButton jBtnSaveImage = new JButton();
		JButton jBtnHelp = new JButton();
		ImageIcon imgProjOpen;
		ImageIcon imgRefresh;
		ImageIcon imgPic;
		ImageIcon imgHelp;
		ImageIcon imgNewImg;
		ImageIcon imgEditImg;
		ImageIcon imgFixRot;

		// Jay added the following line for the icon to start
		// the window for adjusting the position of each frame
		// in an image.

		ImageIcon imgAdjustFrames;
		ImageIcon imgSave;

	ImageIcon imgMain;

	BorderLayout blayContentPane = new BorderLayout();

	JPanel jPanelMain = new JPanel();
	GridBagLayout gblayPanelMain = new GridBagLayout();

		JSplitPane jSplitPaneMain = new JSplitPane();
			JPanel jPanelWorkspace = new JPanel();
				CardLayout cardlayPanelWorkspace = new CardLayout(5, 5);
				JScrollPane jScrollPaneImage = new JScrollPane();
					JPanel jPanelPicDisplay = new PicPanel();
				JScrollPane jScrollPaneText = new JScrollPane();
					JTextArea jTextDisplay = new JTextArea();
			JScrollPane jScrollPaneTreeView = new JScrollPane();

				// Jay changed jTreeFiles to be a ZTTree
				// rather than a JTree.

				ZTTree jTreeFiles =
				  new ZTTree(new File(Res.get("NOPROJ")));
		JLabel statusBar = new JLabel();
			Border borderStatusBar;

	// Jay added the following line to remember which grid image to use.

	String gridImageName;
	ZTImage zti = null;

	// Construct the frame.
	// Jay added the mainArgs argument to get what was passed
	// in from the command line.

	public MainFrame(String[] mainArgs)
	{
		enableEvents(AWTEvent.WINDOW_EVENT_MASK);
		try
		{
			// Jay added the following 20 lines to determine
			// which grid image should be used,
			// based on the first command line argument.

			if (mainArgs.length > 0)
			{ if (mainArgs[0].startsWith("d") == true
			  || mainArgs[0].startsWith("D") == true)
			  { gridImageName = "images/tilediamond.png";
			  }
			  else if (mainArgs[0].startsWith("p") == true
			  || mainArgs[0].startsWith("P") == true)
			  { gridImageName = "images/tilecrosspatch.png";
			  }
			  else if (mainArgs[0].startsWith("c") == true
			  || mainArgs[0].startsWith("C") == true)
			  { gridImageName = "images/tilecross.png";
			  }
			  else
			  { gridImageName = "images/tileoutline.png";
			  }
			}
			else
			{ gridImageName = "images/tileoutline.png";
			}

			doInit();
		}
		catch (Exception e)
		{
			e.printStackTrace();

			// Jay added the following 5 lines so that a popup
			// message occurs for unexpected exceptions.

			Object args[] = { e.toString() };
			JOptionPane.showMessageDialog(this,
			                   Res.format("ERR_UNEXPECTED", args),
			                              Res.get("ERR_ZOOT"),
			                           JOptionPane.ERROR_MESSAGE);
		}

		// Jay added the following 8 lines so that a popup message
		// occurs for unexpected virtual machine errors,
		// such as out of memory.

		catch (VirtualMachineError e)
		{ e.printStackTrace();
		  Object args[] = { e.toString() };
		  JOptionPane.showMessageDialog(this,
		                           Res.format("ERR_UNEXPECTED", args),
		                                Res.get("ERR_ZOOT"),
		                                JOptionPane.ERROR_MESSAGE);
		}
	}

	//Component initialization
	private void doInit() throws Exception
	{
		imgProjOpen = new ImageIcon(com.madsci.zoot.MainFrame.class.getResource(
			"images/openFolder16x16G.png"));
		imgPic = new ImageIcon(com.madsci.zoot.MainFrame.class.getResource(
			"images/pic16x16.png"));
		imgRefresh = new ImageIcon(com.madsci.zoot.MainFrame.class.getResource(
			"images/refresh16x16.png"));
		imgHelp = new ImageIcon(com.madsci.zoot.MainFrame.class.getResource(
			"images/help.png"));
		imgNewImg = new ImageIcon(com.madsci.zoot.MainFrame.class.getResource(
			"images/newImg16x16G.png"));
		imgEditImg = new ImageIcon(com.madsci.zoot.MainFrame.class.getResource(
			"images/editImg16x16G.png"));
		imgFixRot = new ImageIcon(com.madsci.zoot.MainFrame.class.getResource(
			"images/fixRot16x16G.png"));

		// Jay added the following 3 lines to load the icon associated
		// with starting the window for adjusting the position of each
		// frame in an image.

		imgAdjustFrames =
		  new ImageIcon(com.madsci.zoot.MainFrame.class.getResource(
		                                 "images/moveEach16x16.png"));
		imgMain = new ImageIcon(com.madsci.zoot.MainFrame.class.getResource(
			"images/zoot16x16.png"));
		imgSave = new ImageIcon(com.madsci.zoot.MainFrame.class.getResource(
			"images/fileSave16x16.png"));

		this.setSize(new Dimension(600, 500));
		this.setTitle(Res.get("PRODUCTLONG"));

		this.setIconImage(imgMain.getImage());

		//
		// Top-level pane
		//
		contentPane = (JPanel)this.getContentPane();
		contentPane.setLayout(blayContentPane);
		contentPane.setPreferredSize(new Dimension(300, 500));
			blayContentPane.setHgap(0);
			blayContentPane.setVgap(0);

		//
		// Work-area of the top-level pane
		//
		jPanelMain.setLayout(gblayPanelMain);
		jPanelMain.setAlignmentX((float)0.5);
		jPanelMain.setAlignmentY((float)0.5);
		jPanelMain.setBorder(null);

		//
		// Menu items
		//
		jMenuFile.setText(Res.get("MENUFILE"));
		jMenuFile.setMnemonic(KeyEvent.VK_F);
			jMenuFileExit.setText(Res.get("MENUFILEEXIT"));
				jMenuFileExit.setMnemonic(KeyEvent.VK_X);
				jMenuFileExit.setActionCommand(MENUFILEEXIT);
				jMenuFileExit.addActionListener(new MainFrame_jMenuFile_actionAdapter(this));
				jMenuFileExit.setToolTipText(Res.get("TIPMENUFILEEXIT"));
			jMenuFileOpenProj.setText(Res.get("MENUFILEOPENPROJ"));
				jMenuFileOpenProj.setMnemonic(KeyEvent.VK_O);
				jMenuFileOpenProj.setIcon(imgProjOpen);
				jMenuFileOpenProj.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
				jMenuFileOpenProj.setActionCommand(MENUFILEOPENPROJ);
				jMenuFileOpenProj.addActionListener(new MainFrame_jMenuFile_actionAdapter(this));
				jMenuFileOpenProj.setToolTipText(Res.get("TIPMENUFILEOPENPROJ"));
			jMenuFileRefresh.setText(Res.get("MENUFILEREFRESH"));
				jMenuFileRefresh.setMnemonic(KeyEvent.VK_R);
				jMenuFileRefresh.setIcon(imgRefresh);
				jMenuFileRefresh.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F5, ActionEvent.CTRL_MASK));
				jMenuFileRefresh.setActionCommand(MENUFILEREFRESH);
				jMenuFileRefresh.addActionListener(new MainFrame_jMenuFile_actionAdapter(this));
				jMenuFileRefresh.setToolTipText(Res.get("TIPMENUFILEREFRESH"));
		jMenuImage.setText(Res.get("MENUIMAGE"));
		jMenuImage.setMnemonic(KeyEvent.VK_I);
			jMenuImageNewImg.setText(Res.get("MENUIMAGENEWIMG"));
				jMenuImageNewImg.setMnemonic(KeyEvent.VK_N);
				jMenuImageNewImg.setIcon(imgNewImg);
				jMenuImageNewImg.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
				jMenuImageNewImg.setActionCommand(MENUIMAGENEWIMG);
				jMenuImageNewImg.addActionListener(new MainFrame_jMenuImage_actionAdapter(this));
				jMenuImageNewImg.setToolTipText(Res.get("TIPMENUIMAGENEWIMG"));
			jMenuImageEditImg.setText(Res.get("MENUIMAGEEDITIMG"));
				jMenuImageEditImg.setMnemonic(KeyEvent.VK_E);
				jMenuImageEditImg.setIcon(imgEditImg);
				jMenuImageEditImg.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, ActionEvent.CTRL_MASK));
				jMenuImageEditImg.setActionCommand(MENUIMAGEEDITIMG);
				jMenuImageEditImg.addActionListener(new MainFrame_jMenuImage_actionAdapter(this));
				jMenuImageEditImg.setToolTipText(Res.get("TIPMENUIMAGEEDITIMG"));
			jMenuImageFixRot.setText(Res.get("MENUIMAGEFIXROT"));
				jMenuImageFixRot.setMnemonic(KeyEvent.VK_F);
				jMenuImageFixRot.setIcon(imgFixRot);
				jMenuImageFixRot.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, ActionEvent.CTRL_MASK));
				jMenuImageFixRot.setActionCommand(MENUIMAGEFIXROT);
				jMenuImageFixRot.addActionListener(new MainFrame_jMenuImage_actionAdapter(this));
				jMenuImageFixRot.setToolTipText(Res.get("TIPMENUIMAGEFIXROT"));

			// Jay added the following 15 lines to add a menu
			// entry for starting the window for adjusting
			// the position of each frame in an image.

			jMenuImageAdjustFrames.setText(
			  Res.get("MENUIMAGEADJUSTFRAMES"));
				jMenuImageAdjustFrames.setMnemonic(
				  KeyEvent.VK_A);
				jMenuImageAdjustFrames.setIcon(
				  imgAdjustFrames);
				jMenuImageAdjustFrames.setAccelerator(
				  KeyStroke.getKeyStroke(
				    KeyEvent.VK_A, ActionEvent.CTRL_MASK));
				jMenuImageAdjustFrames.setActionCommand(
				  MENUIMAGEADJUSTFRAMES);
				jMenuImageAdjustFrames.addActionListener(
				new MainFrame_jMenuImage_actionAdapter(this));
				jMenuImageAdjustFrames.setToolTipText(
				  Res.get("TIPMENUIMAGEADJUSTFRAMES"));
			jMenuImageSave.setText(Res.get("MENUIMAGESAVEIMG"));
				jMenuImageSave.setMnemonic(KeyEvent.VK_S);
				jMenuImageSave.setIcon(imgSave);
				jMenuImageSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
				jMenuImageSave.setActionCommand(MENUIMAGESAVE);
				jMenuImageSave.addActionListener(new MainFrame_jMenuImage_actionAdapter(this));
				jMenuImageSave.setToolTipText(Res.get("TIPMENUIMAGESAVEIMG"));
		jMenuHelp.setText(Res.get("MENUHELP"));
		jMenuHelp.setMnemonic(KeyEvent.VK_H);

			// Jay added the following 9 lines to add a menu entry
			// for help for the window.

			jMenuHelpForWindow.setIcon(imgHelp);
			jMenuHelpForWindow.setText(
			  Res.get("MENUHELPFORWINDOW"));
			jMenuHelpForWindow.setMnemonic(KeyEvent.VK_H);
			jMenuHelpForWindow.setAccelerator(
			  KeyStroke.getKeyStroke(KeyEvent.VK_H,
			                         ActionEvent.CTRL_MASK));
			jMenuHelpForWindow.addActionListener(
			  new MainFrame_jBtnHelp_ActionAdapter(this));
			jMenuHelpAbout.setText(Res.get("MENUHELPABOUT"));
			jMenuHelpAbout.setMnemonic(KeyEvent.VK_A);
			jMenuHelpAbout.addActionListener(new MainFrame_jMenuHelpAbout_ActionAdapter(this));

		jMenuBarMain.setBorder(null);
		jMenuFile.add(jMenuFileOpenProj);
		jMenuFile.add(jMenuFileRefresh);
		jMenuFile.add(jMenuFileExit);
		jMenuImage.add(jMenuImageNewImg);
		jMenuImage.add(jMenuImageEditImg);
		jMenuImage.add(jMenuImageFixRot);

		// Jay added the following line to add the menu entry
		// for starting the window for adjusting the position of each
		// frame in an image.

		jMenuImage.add(jMenuImageAdjustFrames);
		jMenuImage.add(jMenuImageSave);

		// Jay added the following line to add a menu entry for help
		// for the window.

		jMenuHelp.add(jMenuHelpForWindow);
		jMenuHelp.add(jMenuHelpAbout);
		jMenuBarMain.add(jMenuFile);
		jMenuBarMain.add(jMenuImage);
		jMenuBarMain.add(Box.createHorizontalGlue()); // Flushes the next item to the right!
		jMenuBarMain.add(jMenuHelp);
		this.setJMenuBar(jMenuBarMain);

		//
		// Toolbar
		//
		jBtnOpenProject.setIcon(imgProjOpen);
		jBtnOpenProject.setMargin(new Insets(0, 0, 0, 0));
		jBtnOpenProject.setToolTipText(Res.get("TIPMENUFILEOPENPROJ"));
		jBtnOpenProject.setActionCommand(MENUFILEOPENPROJ);
		jBtnOpenProject.addActionListener(new
		        MainFrame_jMenuFile_actionAdapter(this));

		jBtnRefresh.setIcon(imgRefresh);
		jBtnRefresh.setMargin(new Insets(0, 0, 0, 0));
		jBtnRefresh.setToolTipText(Res.get("TIPMENUFILEREFRESH"));
		jBtnRefresh.setActionCommand(MENUFILEREFRESH);
		jBtnRefresh.addActionListener(new
		        MainFrame_jMenuFile_actionAdapter(this));

		jBtnNewImg.setIcon(imgNewImg);
		jBtnNewImg.setMargin(new Insets(0, 0, 0, 0));
		jBtnNewImg.setToolTipText(Res.get("TIPMENUIMAGENEWIMG"));
		jBtnNewImg.setActionCommand(MENUIMAGENEWIMG);
		jBtnNewImg.addActionListener(new
		        MainFrame_jMenuImage_actionAdapter(this));

		jBtnEditImg.setIcon(imgEditImg);
		jBtnEditImg.setMargin(new Insets(0, 0, 0, 0));
		jBtnEditImg.setToolTipText(Res.get("TIPMENUIMAGEEDITIMG"));
		jBtnEditImg.setActionCommand(MENUIMAGEEDITIMG);
		jBtnEditImg.addActionListener(new
		        MainFrame_jMenuImage_actionAdapter(this));

		jBtnFixRot.setIcon(imgFixRot);
		jBtnFixRot.setMargin(new Insets(0, 0, 0, 0));
		jBtnFixRot.setToolTipText(Res.get("TIPMENUIMAGEFIXROT"));
		jBtnFixRot.setActionCommand(MENUIMAGEFIXROT);
		jBtnFixRot.addActionListener(new
		        MainFrame_jMenuImage_actionAdapter(this));

		// Jay added the following 7 lines to add a button
		// for starting the window for adjusting the position
		// of each frame in an image.

		jBtnAdjustFrames.setIcon(imgAdjustFrames);
		jBtnAdjustFrames.setMargin(new Insets(0, 0, 0, 0));
		jBtnAdjustFrames.setToolTipText(
		                         Res.get("TIPMENUIMAGEADJUSTFRAMES"));
		jBtnAdjustFrames.setActionCommand(MENUIMAGEADJUSTFRAMES);
		jBtnAdjustFrames.addActionListener(new
		        MainFrame_jMenuImage_actionAdapter(this));

		jBtnSaveImage.setIcon(imgSave);
		jBtnSaveImage.setMargin(new Insets(0, 0, 0, 0));
		jBtnSaveImage.setToolTipText(Res.get("TIPMENUIMAGESAVEIMG"));
		jBtnSaveImage.setActionCommand(MENUIMAGESAVE);
		jBtnSaveImage.addActionListener(new
		        MainFrame_jMenuImage_actionAdapter(this));

		jBtnHelp.setIcon(imgHelp);
		jBtnHelp.setMargin(new Insets(0, 0, 0, 0));

		// Jay changed the following to use MENUHELPFORWINDOW instead
		// of TIPMENUHELP, since the help button is different
		// than the About.

		jBtnHelp.setToolTipText(Res.get("MENUHELPFORWINDOW"));

		// Jay added the following 2 lines to listen for the Help
		// button being clicked.

		jBtnHelp.addActionListener(
		  new MainFrame_jBtnHelp_ActionAdapter(this));

		jToolBar.setEnabled(true);
		jToolBar.setBorder(BorderFactory.createEtchedBorder());
		jToolBar.setFloatable(false);
		jToolBar.setRollover(true);
		jToolBar.add(jBtnOpenProject);
		jToolBar.add(jBtnRefresh);
		jToolBar.addSeparator(new Dimension(10, 20));
		jToolBar.add(jBtnNewImg);
		jToolBar.add(jBtnEditImg);
		jToolBar.add(jBtnFixRot);

		// Jay added the following line to add the button for starting
		// the window for adjusting the position of each frame
		// in an image.

		jToolBar.add(jBtnAdjustFrames);
		jToolBar.add(jBtnSaveImage);

		// Jay added the following line to separate the help button
		// slightly from the rest of the buttons.

		jToolBar.addSeparator(new Dimension(10, 20));

		// Jay put the following line back in so that there would be
		// a help button.

		jToolBar.add(jBtnHelp);

		//
		// Status bar
		//
		borderStatusBar = BorderFactory.createBevelBorder(BevelBorder.LOWERED,
												  Color.white,
												  new Color(228, 228, 234),
												  new Color(78, 78, 79),
												  new Color(112, 112, 114));
		statusBar.setBorder(borderStatusBar);
		statusBar.setText(" ");

		//
		// Split pane inside the work area
		//
		jSplitPaneMain.setOrientation(JSplitPane.HORIZONTAL_SPLIT);
		jSplitPaneMain.setBorder(null);
		jSplitPaneMain.setBottomComponent(null);
		jSplitPaneMain.setResizeWeight(0.25);
		jSplitPaneMain.setRightComponent(jPanelWorkspace);

		//
		// Right-side panel in split pane
		//
		jPanelWorkspace.setLayout(cardlayPanelWorkspace);
		jPanelWorkspace.setBorder(BorderFactory.createLoweredBevelBorder());

		jPanelPicDisplay.setBorder(null);
		jPanelPicDisplay.setMaximumSize(new Dimension(100, 100));
		jPanelPicDisplay.setMinimumSize(new Dimension(100, 100));
		jPanelPicDisplay.setDoubleBuffered(true);
		jPanelPicDisplay.setPreferredSize(new Dimension(100, 100));

		jScrollPaneImage.setViewport(null);
		jScrollPaneImage.setBorder(null);
		jScrollPaneImage.setDoubleBuffered(true);
		jScrollPaneImage.setPreferredSize(new Dimension(100, 100));
		jScrollPaneImage.setViewportView(jPanelPicDisplay);

		jTextDisplay.setBorder(null);

		jScrollPaneText.setViewport(null);
		jScrollPaneText.setBorder(null);
		jScrollPaneText.setDoubleBuffered(true);
		jScrollPaneText.setPreferredSize(new Dimension(100, 100));
		jScrollPaneText.setViewportView(jTextDisplay);

		jScrollPaneTreeView.setBorder(BorderFactory.createLoweredBevelBorder());
		jTreeFiles.setAutoscrolls(false);
		jTreeFiles.setShowsRootHandles(false);
		//
		// Set a selection listener on the tree
		//
		jTreeFiles.getSelectionModel().setSelectionMode
		        (TreeSelectionModel.SINGLE_TREE_SELECTION);
		jTreeFiles.addTreeSelectionListener(new TreeSelectionListener()
		{
		    public void valueChanged(TreeSelectionEvent e)
		    {
		        ZTFSTreeModel.PseudoFile node = (ZTFSTreeModel.PseudoFile)
		                           jTreeFiles.getLastSelectedPathComponent();
				ZTFSTreeModel tm = (ZTFSTreeModel)jTreeFiles.getModel();

		        if (node == null) return;

		        if (node.isLeaf() && node.toString().lastIndexOf('.') == -1)
		        {
			        //
			        // If the selected file doesn't have an extension,
			        // let's try to load it into a ZTImage. If we succeed,
			        // display it in the main window.
			        //
					try
					{
						// Since this section
						// can potentially use a lot
						// of resources, Jay added
						// the following 3 lines
						// to free up any used
						// resources.

						zti = null;
						System.runFinalization();
						System.gc();
						zti = new ZTImage(tm.getRootPath(), tm.getRelativePath(node), 0);
						zti.load();
						int rgb = 0xFFFF00FF; // Opaque neon pink
						zti.setTransparentColor(rgb);

						ImageVector iv = new ImageVector();
						for (int i = 0; i < zti.getNumFrames(); i++)
						        iv.add(zti.toBufferedImage(i));

						//
						// Switch the cardlayout's card
						//
						CardLayout cl = (CardLayout)jPanelWorkspace.getLayout();
						cl.show(jPanelWorkspace, CARDIMG);

						//
						// I really hate this
						//
						jMenuImageEditImg.setEnabled(true);
						jMenuImageFixRot.setEnabled(true);
						jBtnEditImg.setEnabled(true);
						jBtnFixRot.setEnabled(true);

						// Jay added the following 4
						// lines to enable the button
						// and menu item for starting
						// the window for adjusting
						// the position of each frame
						// in an image.

						jMenuImageAdjustFrames
						.setEnabled(true);
						jBtnAdjustFrames.setEnabled(
						                        true);
						jMenuImageSave.setEnabled(true);
						jBtnSaveImage.setEnabled(true);

						((PicPanel)jPanelPicDisplay).setImage(iv, zti.getNumFrames() == 1 ? 0 : zti.getFrameRate());
						repaint();
						SwingUtilities.invokeLater(new Thread() { public void run() { System.gc(); } } );
						return;
					}
					catch (Exception e1)
					{
						//
						// Not a ZT image...
						//
					}

					// Jay added the following 8 lines
					// so that a popup message occurs
					// for unexpected virtual machine
					// errors, such as out of memory.

					catch (VirtualMachineError e1)
					{ e1.printStackTrace();
					  Object args[] = { e1.toString() };
					  JOptionPane.showMessageDialog(null,
					   Res.format("ERR_UNEXPECTED", args),
					                  Res.get("ERR_ZOOT"),
					           JOptionPane.ERROR_MESSAGE);
					}
		        }
			    else if (node.isLeaf() && ZootUtils.isTextFile(node.toString()))
			    {
				    try
				    {
					    //
					    // Open the file and display the text
					    //
					    File f = new File(node.getPath());
				        FileReader fr = new FileReader(f);
					    char cBuf[] = new char[(int)f.length()];
					    fr.read(cBuf, 0, (int)f.length());

					    jTextDisplay.setEditable(false);
					    jTextDisplay.setText(new String(cBuf));
					    jTextDisplay.setCaretPosition(0);

					    //
					    // Switch the cardlayout's card
					    //
					    CardLayout cl = (CardLayout)jPanelWorkspace.getLayout();
					    cl.show(jPanelWorkspace, CARDTXT);

					    jMenuImageEditImg.setEnabled(false);
					    jMenuImageFixRot.setEnabled(false);
					    jBtnEditImg.setEnabled(false);
					    jBtnFixRot.setEnabled(false);

					    // Jay added the following 4 lines
					    // to disable the button and menu
					    // item for starting the window
					    // for adjusting the position
					    // of each frame in an image.

					    jMenuImageAdjustFrames.setEnabled(
					                               false);
					    jBtnAdjustFrames.setEnabled(
					                               false);
					    jMenuImageSave.setEnabled(false);
					    jBtnSaveImage.setEnabled(false);

					    return;
				    }
				    catch (IOException ioe)
				    {
				    }
			    }

				//
				// Not an interesting node... Clear the display area
				//
				zti = null;
				((PicPanel)jPanelPicDisplay).setImage(null, 0);

			    //
			    // Switch the cardlayout's card
			    //
			    CardLayout cl = (CardLayout)jPanelWorkspace.getLayout();
			    cl.show(jPanelWorkspace, CARDIMG);

			    jMenuImageEditImg.setEnabled(false);
			    jMenuImageFixRot.setEnabled(false);
			    jBtnEditImg.setEnabled(false);
			    jBtnFixRot.setEnabled(false);

			    // Jay added the following 2 lines to disable
			    // the button and menu item for starting
			    // the window for adjusting the position of each
			    // frame in an image.

			    jMenuImageAdjustFrames.setEnabled(false);
			    jBtnAdjustFrames.setEnabled(false);
			    jMenuImageSave.setEnabled(false);
			    jBtnSaveImage.setEnabled(false);

				repaint();
		    }
		});

		contentPane.add(jToolBar, BorderLayout.NORTH);
		contentPane.add(jPanelMain, BorderLayout.CENTER);
		contentPane.add(statusBar, BorderLayout.SOUTH);

		jPanelMain.add(jSplitPaneMain,
		               new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0,
		                                      GridBagConstraints.CENTER,
		                                      GridBagConstraints.BOTH,
		                                      new Insets(5, 5, 5, 0), 0, 0));
		jSplitPaneMain.add(jPanelWorkspace, JSplitPane.BOTTOM);
		jSplitPaneMain.add(jScrollPaneTreeView, JSplitPane.TOP);
		jScrollPaneTreeView.getViewport().add(jTreeFiles, null);
		jPanelWorkspace.add(jScrollPaneImage, CARDIMG);
		jPanelWorkspace.add(jScrollPaneText, CARDTXT);

		jMenuImageEditImg.setEnabled(false);
		jMenuImageFixRot.setEnabled(false);
		jBtnEditImg.setEnabled(false);
		jBtnFixRot.setEnabled(false);

		// Jay added the following 2 lines to disable the button
		// and menu item for starting the window for adjusting
		// the position of each frame in an image.

		jMenuImageAdjustFrames.setEnabled(false);
		jBtnAdjustFrames.setEnabled(false);
		jMenuImageNewImg.setEnabled(false);
		jBtnNewImg.setEnabled(false);
		jMenuImageSave.setEnabled(false);
		jBtnSaveImage.setEnabled(false);

		MenuSelectionManager.defaultManager().addChangeListener(this);
	}

	public void doStart()
	{
		String sLastProj = prefs.get(LASTPROJ, null);
		if (sLastProj == null || !new File(sLastProj).exists())
		{
			//
			// Prompt user to open a project right away
			//
			EventQueue.invokeLater(new WizardRunner(this));
		}
		else
		{ // Jay wrapped this else clause in a try...catch in case
		  // there was an exception or virtual memory error,
		  // such as out of memory, trying to load the last project.
		  // If that happens, a popup will occur for the unexpected
		  // error and the Wizard will be scheduled to run
		  // to allow choosing a different project.

		  try
		  {
			//
			// This populates the tree with the files in the directory
			//
			((ZTTree)jTreeFiles).refresh(new File(sLastProj));
			jTreeFiles.setSelectionRow(0);

			jMenuImageNewImg.setEnabled(true);
			jBtnNewImg.setEnabled(true);

			this.setVisible(true);
			jTreeFiles.requestFocusInWindow();
		  }
		  catch (Exception e)
		  { e.printStackTrace();
		    Object args[] = { e.toString() };
		    JOptionPane.showMessageDialog(this,
		                           Res.format("ERR_UNEXPECTED", args),
		                                  Res.get("ERR_ZOOT"),
		                                  JOptionPane.ERROR_MESSAGE);
		    EventQueue.invokeLater(new WizardRunner(this));
		  }
		  catch (VirtualMachineError e)
		  { e.printStackTrace();
		    Object args[] = { e.toString() };
		    JOptionPane.showMessageDialog(this,
		                           Res.format("ERR_UNEXPECTED", args),
		                                Res.get("ERR_ZOOT"),
		                                JOptionPane.ERROR_MESSAGE);
		    EventQueue.invokeLater(new WizardRunner(this));
		  }
		}
	}

	public boolean isProjectOpen()
	{
		return jTreeFiles.getModel().toString().compareTo(Res.get("NOPROJ")) != 0;
	}

	//File | Exit action performed
	public void jMenuFileExit_actionPerformed(ActionEvent e)
	{
		System.exit(0);
	}

	//Help | About action performed
	public void jMenuHelpAbout_actionPerformed(ActionEvent e)
	{
		MainFrame_AboutBox dlg = new MainFrame_AboutBox(this);
		Dimension dlgSize = dlg.getPreferredSize();
		Dimension frmSize = getSize();
		Point loc = getLocation();
		dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x,
						(frmSize.height - dlgSize.height) / 2 + loc.y);
		dlg.setModal(true);
		dlg.pack();
		dlg.show();
	}

	//Overridden so we can exit when window is closed
	protected void processWindowEvent(WindowEvent e)
	{
		super.processWindowEvent(e);
		if (e.getID() == WindowEvent.WINDOW_CLOSING)
		{
			jMenuFileExit_actionPerformed(null);
		}
	}

	void jMenuFileOpenProj_actionPerformed(ActionEvent e)
	{	// Since this action can potentially use a lot of resources,
		// Jay added the following 2 lines to free up any used
		// resources.

		System.runFinalization();
		System.gc();

//TODO: Open a ZTD file, extract to tmp dir and show that dir instead...
		//
		// Ask the user to open a project (i.e. a directory)
		//
		String sTopDir = getLastProject();
		sTopDir += File.separator;

		JFileChooser jfc = new JFileChooser();
		jfc.setDialogTitle(Res.get("DLGOPENPROJ"));
		jfc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		jfc.setCurrentDirectory(new File(sTopDir));

		int nRet = jfc.showOpenDialog(this);

		if (nRet != JFileChooser.APPROVE_OPTION)
			return;

		File f = jfc.getSelectedFile();

		try
		{
			//
			// Note that we indirectly check for existence of the file here...
			//
			setLastProject(f.getCanonicalPath());
		}
		catch (IOException e1) { return; }

		//
		// This populates the tree with the files in the directory
		//
		((ZTTree)jTreeFiles).refresh(f);
		jTreeFiles.setSelectionRow(0);
		jTreeFiles.requestFocusInWindow();

		jMenuImageNewImg.setEnabled(true);
		jBtnNewImg.setEnabled(true);
	}

	void jMenuFileRefresh_actionPerformed(ActionEvent e)
	{	// Since this action can potentially use a lot of resources,
		// Jay added the following 2 lines to free up any used
		// resources.

		System.runFinalization();
		System.gc();
		((ZTTree)jTreeFiles).refresh();
		jTreeFiles.setSelectionRow(0);
	}

	void jMenuImageNewImg_actionPerformed(ActionEvent e)
	{	// Since this action can potentially use a lot of resources,
		// Jay added the following 2 lines to free up any used
		// resources.

		System.runFinalization();
		System.gc();

		if (!isProjectOpen())
		{
			JOptionPane.showMessageDialog(this, Res.get("ERR_NOPROJ"),
			                              Res.get("ERR_ZOOT"), JOptionPane.ERROR_MESSAGE);
			return;
		}

		//
		// Bring up a File Save dialog
		//
		String sTopDir = ((ZTFSTreeModel)(jTreeFiles.getModel())).getRootPath();
		sTopDir += File.separator;

		JFileChooser jfc = new JFileChooser();
		jfc.setDialogTitle(Res.get("DLGNEWIMAGE"));
		jfc.setCurrentDirectory(new File(sTopDir));
		jfc.setApproveButtonText(Res.get("DLGNEWIMAGEOKBTNTEXT"));
		jfc.setApproveButtonMnemonic(Res.get("DLGNEWIMAGEOKBTNMNEMONIC").charAt(0));
		jfc.setApproveButtonToolTipText(Res.get("TIPDLGNEWIMAGEOKBTN"));
		int nRet = jfc.showOpenDialog(this);

		String sFile = null;
		File f = null;
		if (nRet == JFileChooser.APPROVE_OPTION)
		{
			f = jfc.getSelectedFile();
			setLastVisitedDir(f.getParent());
			sFile = f.getPath();
		}

		if (sFile == null)
			return;

		//
		// Make sure file doesn't already exist!
		//
		if (f.exists())
		{
			JOptionPane.showMessageDialog(this, Res.get("ERR_FILEEXISTS"),
			                              Res.get("ERR_ZOOT"), JOptionPane.ERROR_MESSAGE);
			EventQueue.invokeLater(new Thread() { public void run() {jMenuImageNewImg_actionPerformed(null);} } );
			return;
		}

		String sFullNodePath = f.getPath();
		String sRootPath = ((ZTFSTreeModel.PseudoFile)(jTreeFiles.getModel().getRoot())).sPath;

		if (sRootPath.charAt(sRootPath.length()-1) != File.separatorChar)
			sRootPath += File.separator;

		if (sFullNodePath.indexOf(sRootPath) != 0)
		{
			JOptionPane.showMessageDialog(this, Res.get("ERR_BADPROJECTROOT"),
			                              Res.get("ERR_ZOOT"), JOptionPane.ERROR_MESSAGE);
			EventQueue.invokeLater(new Thread() { public void run() {jMenuImageNewImg_actionPerformed(null);} } );
			return;
		}

		String sRelPath = sFullNodePath.substring(sRootPath.length());

		zti = new ZTImage(sRootPath, sRelPath, 0x3E8);
		jMenuImageEditImg_actionPerformed(null);
	}

	void jMenuImageEditImg_actionPerformed(ActionEvent e)
	{	// Since this action can potentially use a lot of resources,
		// Jay added the following 2 lines to free up any used
		// resources.

		System.runFinalization();
		System.gc();

		if (zti == null)
			return;

		//
		// Bring it up in a window
		//
		// Jay wrapped the following section in a "try" so that
		// exceptions and virtual machine errors, such as out
		// of memory, could be caught and shown.  He also changed
		// EditImgDlg to EditZTImage to reflect it changing
		// from a JDialog to a JFrame, which allows it
		// to be maximized.

		try
		{ EditZTImage frf = new EditZTImage(zti, this);
		  frf.pack();
		  centerFrame(frf);
		  frf.show();
		}
		catch (Exception e1)
		{ e1.printStackTrace();
		  Object args[] = { e1.toString() };
		  JOptionPane.showMessageDialog(this,
		                           Res.format("ERR_UNEXPECTED", args),
		                                Res.get("ERR_ZOOT"),
		                                JOptionPane.ERROR_MESSAGE);
		}
		catch (VirtualMachineError e1)
		{ e1.printStackTrace();
		  Object args[] = { e1.toString() };
		  JOptionPane.showMessageDialog(this,
		                           Res.format("ERR_UNEXPECTED", args),
		                                Res.get("ERR_ZOOT"),
		                                JOptionPane.ERROR_MESSAGE);
		}
	}

	void jMenuImageFixRot_actionPerformed(ActionEvent e)
	{	// Since this action can potentially use a lot of resources,
		// Jay added the following 2 lines to free up any used
		// resources.

		System.runFinalization();
		System.gc();

		if (zti == null)
			return;

		// Jay wrapped the following section in a "try" so that
		// exceptions and virtual machine errors, such as out
		// of memory, could be caught and shown.  He also changed
		// from a JDialog FixRotDlg of frame 0 to a JFrame
		// AdjustZTImagePosition of the image as a whole.  The use
		// of JFrame also allows that window to be maximized.

		try
		{ JFrame frf = new AdjustZTImagePosition(zti, false, this);

		//
		// If the image only contains one frame, go to the fix rot dlg directly.
		// Otherwise, bring up the frame selection box.
		//
//		if (zti.getNumFrames() == 1)
//			frf = new FixRotDlg(zti, 0, this);
//		else
//			frf = new FrameSelectionDlg(zti, this);

		  frf.pack();
		  centerFrame(frf);
		  frf.show();
		}
		catch (Exception e1)
		{ e1.printStackTrace();
		  Object args[] = { e1.toString() };
		  JOptionPane.showMessageDialog(this,
		                           Res.format("ERR_UNEXPECTED", args),
		                                Res.get("ERR_ZOOT"),
		                                JOptionPane.ERROR_MESSAGE);
		}
		catch (VirtualMachineError e1)
		{ e1.printStackTrace();
		  Object args[] = { e1.toString() };
		  JOptionPane.showMessageDialog(this,
		                           Res.format("ERR_UNEXPECTED", args),
		                                Res.get("ERR_ZOOT"),
		                                JOptionPane.ERROR_MESSAGE);
		}
	}

	// Jay added the following method to start the window that allows
	// adjusting the position of each frame in an image.

	void jMenuImageAdjustFrames_actionPerformed(ActionEvent e)
	{ System.runFinalization();
	  System.gc();

	  if (zti == null)
	  { return;
	  }

	  try
	  { JFrame frf = new AdjustZTImagePosition(zti, true, this);
	    frf.pack();
	    centerFrame(frf);
	    frf.show();
	  }
	  catch (Exception e1)
	  { e1.printStackTrace();
	    Object args[] = { e1.toString() };
	    JOptionPane.showMessageDialog(this,
	                                  Res.format("ERR_UNEXPECTED", args),
	                                  Res.get("ERR_ZOOT"),
	                                  JOptionPane.ERROR_MESSAGE);
	  }
	  catch (VirtualMachineError e1)
	  { e1.printStackTrace();
	    Object args[] = { e1.toString() };
	    JOptionPane.showMessageDialog(this,
	                                  Res.format("ERR_UNEXPECTED", args),
	                                  Res.get("ERR_ZOOT"),
	                                  JOptionPane.ERROR_MESSAGE);
	  }
	}

	//
	// Save the selected ZTImage to a set of files
	//
	void jMenuImageSave_actionPerformed(ActionEvent e)
	{	// Since this action can potentially use a lot of resources,
		// Jay added the following 2 lines to free up any used
		// resources.

		System.runFinalization();
		System.gc();

		if (zti == null)
			return;

		//
		// Bring up a File Save dialog
		//
		String sFile = selectPNGToSave();
		if (sFile == null)
			return;

		//
		// Make sure the file has a .PNG extension
		//
		if (!zti.isSupportedType(sFile))
			sFile += ".png";

		int nPos = sFile.lastIndexOf(".");

		//
		// Determine the desired format for the target images
		//
		String sBase = sFile.substring(0, nPos);
		String sExt = sFile.substring(nPos + 1);

		//
		// Save the ZT frames into individual images
		//
		for (int i = 0; i < zti.getNumFrames(); i++)
		{
			//
			// Construct <base>XXX.<ext> output file name
			//
			String sOutImage = sBase;
			// Yuck
			if (i < 10)
				sOutImage = sOutImage + "00" + i;
			else if ( i < 100 )
				sOutImage = sOutImage + "0" + i;
			else
				sOutImage = sOutImage + i;
			sOutImage = sOutImage + "." + sExt;

			BufferedImage bi = null;
			try
			{
				bi = zti.toBufferedImage(i);
			}
			catch (ConversionException e1)
			{	// Jay added the following line to provide more
				// information of the exception.

				e1.printStackTrace();
				Object args[] = { sOutImage, e1.toString() };
				JOptionPane.showMessageDialog(this, Res.format("ERR_FILESAVE", args),
				                              Res.get("ERR_ZOOT"), JOptionPane.ERROR_MESSAGE);
			}

			File f2 = new File(sOutImage);
			try
			{
				ImageIO.write(bi, sExt, f2);
			}
			catch (IOException e1)
			{	// Jay added the following line to provide more
				// information of the exception.

				e1.printStackTrace();
				Object args[] = { sOutImage, e1.toString() };
				JOptionPane.showMessageDialog(this, Res.format("ERR_FILESAVE", args),
				                              Res.get("ERR_ZOOT"), JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	// Jay added the following method to return the grid image name
	// that should be used.

	public String getGridImageName()
	{ return gridImageName;
	}

	// Jay added the following method to show the help box
	// via the help button.

	public void help_actionPerformed(ActionEvent e)
	{ MainFrame_HelpBox dlg = new MainFrame_HelpBox(this);
	  Dimension dlgSize = dlg.getPreferredSize();
	  Dimension frmSize = getSize();
	  Point loc = getLocation();

	  dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x,
	                  (frmSize.height - dlgSize.height) / 2 + loc.y);
	  dlg.pack();
	  dlg.show();
	}

	// Jay added the following method to allow refreshing
	// the tree in case an image was changed by another class.

	public void refresh()
	{ try
	  {
	    // Since the refresh will also collapse the tree, it needs to be
	    // reexpanded.  The refresh will also create a new root node,
	    // so that has to be used in reexpanding and reselecting paths.

	    TreePath selectPath = jTreeFiles.getSelectionPath();
	    Object[] selectComponents = null;

	    if (selectPath != null)
	    { selectComponents = selectPath.getPath();
	    }

	    TreePath oldRoot = jTreeFiles.getPathForRow(0);
	    Enumeration expand = jTreeFiles.getExpandedDescendants(oldRoot);
	    jTreeFiles.refresh();
	    jTreeFiles.setExpandsSelectedPaths(true);
	    Object rootComponent =
	      jTreeFiles.getPathForRow(0).getPathComponent(0);

	    // The following reexpands every path that was expanded prior
	    // to the refresh.  Since the root node changed, the root node
	    // in the path has to be set to the new root node.

	    while (expand.hasMoreElements())
	    { Object[] components =
	        ((TreePath)(expand.nextElement())).getPath();
	      components[0] = rootComponent;
	      jTreeFiles.expandPath(new TreePath(components));
	    }

	    // The following reselects and makes visible the path that was
	    // selected before the refresh.  Its root node has changed.

	    if (selectPath != null)
	    { selectComponents[0] = rootComponent;
	      TreePath tree = new TreePath(selectComponents);
	      jTreeFiles.setSelectionPath(tree);
	      jTreeFiles.scrollPathToVisible(tree);
	    }
	  }
	  catch (Exception e1)
	  { e1.printStackTrace();
	    Object args[] = { e1.toString() };
	    JOptionPane.showMessageDialog(this,
	                                  Res.format("ERR_UNEXPECTED", args),
	                                  Res.get("ERR_ZOOT"),
	                                  JOptionPane.ERROR_MESSAGE);
	  }
	  catch (VirtualMachineError e1)
	  { e1.printStackTrace();
	    Object args[] = { e1.toString() };
	    JOptionPane.showMessageDialog(this,
	                                  Res.format("ERR_UNEXPECTED", args),
	                                  Res.get("ERR_ZOOT"),
	                                  JOptionPane.ERROR_MESSAGE);
	  }
	}

	public String selectPNGToSave()
	{
		String sTopDir = getLastVisitedDir();
		sTopDir += File.separator;

		JFileChooser jfc = new JFileChooser();
		jfc.setDialogTitle(Res.get("DLGSAVEIMAGE"));
		jfc.setCurrentDirectory(new File(sTopDir));
		jfc.addChoosableFileFilter(new PNGFileFilter());
		jfc.setAcceptAllFileFilterUsed(false);
		int nRet = jfc.showSaveDialog(this);

		String sFile = null;
		if (nRet == JFileChooser.APPROVE_OPTION)
		{
			File f = jfc.getSelectedFile();
			setLastVisitedDir(f.getParent());
			sFile = f.getPath();
		}
		return sFile;
	}

	/**
	 * Center the frame on the screen.
	 * @param frame     The frame to center
	 */
	public static void centerFrame(Component frame)
	{
		//Center the window
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension frameSize = frame.getSize();
		if (frameSize.height > screenSize.height)
		{
			frameSize.height = screenSize.height;
		}
		if (frameSize.width > screenSize.width)
		{
			frameSize.width = screenSize.width;
		}
		frame.setLocation((screenSize.width - frameSize.width) / 2,
						  (screenSize.height - frameSize.height) / 2);
	}

	public String getLastVisitedDir()
	{
		String sDef = System.getProperty("user.dir");
		String sDir = prefs.get(LASTDIR, sDef);
		if (!new File(sDir).exists())
			sDir = sDef;
		return sDir;
	}

	public void setLastVisitedDir(String sDir)
	{
		prefs.put(LASTDIR, sDir);
	}

	public String getLastProject()
	{
		String sDef = System.getProperty("user.dir");
		String sProj = prefs.get(LASTPROJ, sDef);
		if (!new File(sProj).exists())
			sProj = sDef;
		return sProj;
	}

	public void setLastProject(String sDir)
	{
		prefs.put(LASTPROJ, sDir);
	}

	/**
	 * Update the statusbar's text when a menu item is selected
	 * @param e
	 */
	public void stateChanged(ChangeEvent e)
	{
		try
		{
			MenuSelectionManager msm = (MenuSelectionManager)e.getSource();
			MenuElement me[] = msm.getSelectedPath();
			Component c = me[me.length-1].getComponent();
			if (c instanceof JMenuItem)
			{
				JMenuItem jme = (JMenuItem)c;
				String s = jme.getToolTipText();
				if (s == null)
					s = " ";
				statusBar.setText(s);
			}
		}
		catch (Exception ex)
		{
			statusBar.setText(" ");
			return;
		}
	}
}

class MainFrame_jMenuFile_actionAdapter
	implements java.awt.event.ActionListener
{
	MainFrame adaptee;

	MainFrame_jMenuFile_actionAdapter(MainFrame adaptee)
	{
		this.adaptee = adaptee;
	}

	public void actionPerformed(ActionEvent e)
	{
		if (e.getActionCommand().compareTo(adaptee.MENUFILEEXIT) == 0)
			adaptee.jMenuFileExit_actionPerformed(e);
		else if (e.getActionCommand().compareTo(adaptee.MENUFILEOPENPROJ) == 0)
		{ // Jay wrapped the actionPerformed in a try...catch so that
		  // a popup message occurs for unexpected exceptions
		  // or virtual machine errors, such as out of memory.

		  try
		  { adaptee.jMenuFileOpenProj_actionPerformed(e);
		  }
		  catch (Exception e1)
		  { e1.printStackTrace();
		    Object args[] = { e1.toString() };
		    JOptionPane.showMessageDialog(adaptee,
		                           Res.format("ERR_UNEXPECTED", args),
		                                  Res.get("ERR_ZOOT"),
		                                  JOptionPane.ERROR_MESSAGE);
		  }
		  catch (VirtualMachineError e1)
		  { e1.printStackTrace();
		    Object args[] = { e1.toString() };
		    JOptionPane.showMessageDialog(adaptee,
		                           Res.format("ERR_UNEXPECTED", args),
		                                  Res.get("ERR_ZOOT"),
		                                  JOptionPane.ERROR_MESSAGE);
		  }
		}
		else if (e.getActionCommand().compareTo(adaptee.MENUFILEREFRESH) == 0)
		{ // Jay wrapped the actionPerformed in a try...catch so that
		  // a popup message occurs for unexpected exceptions
		  // or virtual machine errors, such as out of memory.

		  try
		  { adaptee.jMenuFileRefresh_actionPerformed(e);
		  }
		  catch (Exception e1)
		  { e1.printStackTrace();
		    Object args[] = { e1.toString() };
		    JOptionPane.showMessageDialog(adaptee,
		                           Res.format("ERR_UNEXPECTED", args),
		                                  Res.get("ERR_ZOOT"),
		                                  JOptionPane.ERROR_MESSAGE);
		  }
		  catch (VirtualMachineError e1)
		  { e1.printStackTrace();
		    Object args[] = { e1.toString() };
		    JOptionPane.showMessageDialog(adaptee,
		                           Res.format("ERR_UNEXPECTED", args),
		                                  Res.get("ERR_ZOOT"),
		                                  JOptionPane.ERROR_MESSAGE);
		  }
		}
	}
}

class MainFrame_jMenuImage_actionAdapter extends AbstractAction
{
	MainFrame adaptee;

	MainFrame_jMenuImage_actionAdapter(MainFrame adaptee)
	{
		this.adaptee = adaptee;
	}

	public boolean isEnabled()
	{
		return adaptee.isProjectOpen();
	}

	public void actionPerformed(ActionEvent e)
	{ // Jay wrapped this method in a try...catch so that
	  // a popup message occurs for unexpected exceptions
	  // or virtual machine errors, such as out of memory.

	  try
	  {
		if (e.getActionCommand().compareTo(adaptee.MENUIMAGENEWIMG) == 0)
			adaptee.jMenuImageNewImg_actionPerformed(e);
		else if (e.getActionCommand().compareTo(adaptee.MENUIMAGEEDITIMG) == 0)
			adaptee.jMenuImageEditImg_actionPerformed(e);
		else if (e.getActionCommand().compareTo(adaptee.MENUIMAGEFIXROT) == 0)
			adaptee.jMenuImageFixRot_actionPerformed(e);

		// Jay added the following 4 lines to start the window
		// that allows adjusting the position of each frame
		// in an image.

		else if (e.getActionCommand()
                          .compareTo(adaptee.MENUIMAGEADJUSTFRAMES) == 0)
		{ adaptee.jMenuImageAdjustFrames_actionPerformed(e);
		}
		else if (e.getActionCommand().compareTo(adaptee.MENUIMAGESAVE) == 0)
			adaptee.jMenuImageSave_actionPerformed(e);
	  }
	  catch (Exception e1)
	  { e1.printStackTrace();
	    Object args[] = { e1.toString() };
	    JOptionPane.showMessageDialog(adaptee,
	                                  Res.format("ERR_UNEXPECTED", args),
	                                  Res.get("ERR_ZOOT"),
	                                  JOptionPane.ERROR_MESSAGE);
	  }
	  catch (VirtualMachineError e1)
	  { e1.printStackTrace();
	    Object args[] = { e1.toString() };
	    JOptionPane.showMessageDialog(adaptee,
	                                  Res.format("ERR_UNEXPECTED", args),
	                                  Res.get("ERR_ZOOT"),
	                                  JOptionPane.ERROR_MESSAGE);
	  }
	}
}

class MainFrame_jMenuHelpAbout_ActionAdapter
	implements ActionListener
{
	MainFrame adaptee;

	MainFrame_jMenuHelpAbout_ActionAdapter(MainFrame adaptee)
	{
		this.adaptee = adaptee;
	}

	public void actionPerformed(ActionEvent e)
	{
		adaptee.jMenuHelpAbout_actionPerformed(e);
	}
}

// Jay added the following class to listen for the Help button being pressed.

class MainFrame_jBtnHelp_ActionAdapter implements ActionListener
{ MainFrame adaptee;

  MainFrame_jBtnHelp_ActionAdapter(MainFrame adaptee)
  { this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  { adaptee.help_actionPerformed(e);
  }
}

class PNGFileFilter extends javax.swing.filechooser.FileFilter
{
	public String getDescription()
	{
		return Res.get("PNGFILEDESC");
	}

	public boolean accept(File f)
	{
	    if (f.isDirectory())
			return true;

		if (ZTImage.isSupportedType(f.getName()))
			return true;

	    return false;
	}
}

class WizardRunner implements Runnable
{
	Frame parent;

	public WizardRunner(Frame owner)
	{
		parent = owner;
	}

	public void run()
	{
		//
		// Create and show the wizard page
		//
		WizardPage frf = new WizardPage(parent);
		frf.pack();
		MainFrame.centerFrame(frf);
		frf.setModal(true);
		frf.show();

		//
		// Done with the wizard; show the main frame
		//
		parent.setVisible(true);
	}
}

/********************************************************************
 *
 * $Log: /Zoot/src/com/madsci/zoot/MainFrame.java $
 *
 * 14   12/27/09 11:57p Jay
 * Show unexpected errors, finish adding help button, ask for another
 * project if the last one cannot be started, have Java free unused resources
 * sooner, call the rotation adjusting version that sets offsets of all
 * frames based on the first, add button and menu item to allow starting
 * a window to adjust the position of each frame in an image, allow other
 * classes to refresh the main tree when an image has changed, and change
 * the ZT Image editing and adjusting windows to be JFrame instead of JDialog.
 *
 * 13    8/11/03 3:43p Fpoulin
 * Added wizard page for new installs.
 *
 * 12    7/16/03 6:48p Fpoulin
 * Use Java Preferences to provide some persistence.
 *
 * 11    7/13/03 12:18p Fpoulin
 * When viewing text file, make first line initially visible.
 *
 * 10    7/13/03 12:13p Fpoulin
 * Added ability to preview text files.
 *
 * 9     7/11/03 1:44a Fpoulin
 * Added "new image" feature
 *
 * 8     7/09/03 7:25p Fpoulin
 * Fixed error reporting dialogs.
 *
 * 7     7/09/03 4:38p Fpoulin
 * Minor refactoring.
 *
 * 6     7/08/03 4:53a Fpoulin
 * Plugged frame edition features.
 *
********************************************************************/
