/////////////////////////////////////////////////////////////////////
//
// PicPanel.java :
//
//  Panel in the main frame that displays a ZT image.
//
/////////////////////////////////////////////////////////////////////
//
// $Revision: 6 $
// $Date: 12/17/09 6:09a $
//
/////////////////////////////////////////////////////////////////////

package com.madsci.zoot;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * Displays a ZT image in the main window
 */
class PicPanel
	extends JPanel
	implements
        MouseWheelListener,
        ChangeListener
{
	// TODO: This shouldn't be here. It should belong to the creating entity -- we should only have a pointer to it here.
	SpinnerNumberModel msm = null;

	ImageVector vecFrames = null;

	int nCurrentFrame = 0;
	Timer timer = null;

	ActionListener taskAnimateFrame = new ActionListener()
	{
		public void actionPerformed(ActionEvent evt)
		{
			//
			// Paint the next frame
			//
			nCurrentFrame++;
			nCurrentFrame %= vecFrames.size();
			repaint();
		}
	};

	PicPanel()
	{
		addMouseWheelListener(this);
		msm = new SpinnerNumberModel(1.0F, 0.99F, 5.01F, 0.2F);
		msm.addChangeListener(this);
	}

	public float getScaleFactor() { return ((Number)(msm.getValue())).floatValue(); }

	public void setImage(ImageVector iv, int delay)
	{
		vecFrames = iv;
		nCurrentFrame = 0;

		recalcSize();

		if (timer != null)
		{
			timer.stop();
			timer = null;
		}

		if (iv != null && iv.size() > 1)
		{
			if (delay > 0)
			{
				timer = new Timer(delay, taskAnimateFrame);
				timer.start();
			}
		}
		getParent().invalidate();
	}

	public void setFramerate(int nNewFramerate)
	{
		setImage(vecFrames, nNewFramerate);
	}

	void recalcSize()
	{
		Dimension dim = new Dimension(10, 10);

		if (vecFrames != null && vecFrames.size() != 0)
		{
			dim = vecFrames.getMaxCellSize();
			dim.setSize(((int)(dim.width * getScaleFactor())) + 10,
			            ((int)(dim.height * getScaleFactor())) + 10);
		}

		setMinimumSize(dim);
		setPreferredSize(dim);
	}

	public void paint(Graphics g)
	{
		super.paint(g);
		if (vecFrames != null && vecFrames.size() != 0)
		{
			int nW = (int)(vecFrames.getImg(nCurrentFrame).getWidth() * getScaleFactor());
			int nH = (int)(vecFrames.getImg(nCurrentFrame).getHeight() * getScaleFactor());
			Rectangle r = getBounds();
			Image img = ((Image)vecFrames.getImg(nCurrentFrame)).getScaledInstance(nW, nH,
				Image.SCALE_DEFAULT);

			// Jay changed the rest of this routine to take
			// the offsets into account so that animations
			// do not jump around for integer scale factors.

			int width0 = vecFrames.getImg(0).getWidth();
			int offsetX0 = vecFrames.getImg(0).getOffsets().x;
			int offsetX =
			  vecFrames.getImg(nCurrentFrame).getOffsets().x;
			int positionX =
			  (int)((r.width - (width0 * getScaleFactor())) / 2)
			  - (int)((offsetX - offsetX0) * getScaleFactor());
			int height0 = vecFrames.getImg(0).getHeight();
			int offsetY0 = vecFrames.getImg(0).getOffsets().y;
			int offsetY =
			  vecFrames.getImg(nCurrentFrame).getOffsets().y;
			int positionY =
			  (int)((r.height - (height0 * getScaleFactor())) / 2)
			  - (int)((offsetY - offsetY0) * getScaleFactor());
			g.drawImage(img, positionX, positionY, null, null);
		}
	}

	/**
	 * Called when the Spinner model has updated the scale value
	 * @param e
	 */
	public void stateChanged(ChangeEvent e)
	{
		recalcSize();
		Component c =  getParent();
		if (c instanceof JSplitPane)
			((JSplitPane)c).resetToPreferredSizes();
		repaint();
	}

	public void mouseWheelMoved(MouseWheelEvent e)
	{
		int nNumTicks = e.getWheelRotation();
		try
		{
			msm.setValue(nNumTicks > 0 ? msm.getNextValue() : msm.getPreviousValue());
		}
		catch (IllegalArgumentException iae) {}
	}
}


/********************************************************************
 *
 * $Log: /Zoot/src/com/madsci/zoot/PicPanel.java $
 *
 * 6    12/17/09 6:09a Jay
 * Take offsets into account when showing animations so that they do not jump
 * around for integer scale factors.
 *
 * 5     7/11/03 1:44a Fpoulin
 * Added "new image" feature
 *
 * 4     7/09/03 7:24p Fpoulin
 * Handle zero-frame images.
 *
 * 3     7/08/03 4:53a Fpoulin
 * Plugged frame edition features.
 *
********************************************************************/
