/////////////////////////////////////////////////////////////////////
//
// ZTFrame.java :
//
//  One ZT image frame.
//
/////////////////////////////////////////////////////////////////////
//
// $Revision: 6 $
// $Date: 11/11/09 3:32p $
//
/////////////////////////////////////////////////////////////////////

package com.madsci.zoot;

import java.awt.*;
import java.awt.image.*;
import java.io.IOException;
import java.nio.*;
import java.util.*;

public class ZTFrame
{
	class PixelSet
	{
		int yNumTrans = 0;
		Vector vecColorPixels = new Vector();
	};

	class Line
	{
		Vector vecPixelSets = new Vector();
	};

	Vector vecBitmap = new Vector();

	private short nWidth;
	private short nHeight;
	private short nOffsetY;
	private short nOffsetX;

	public ZTFrame(short width, short height)
	{
		nWidth   = width;
		nHeight  = height;
		nOffsetX = (short)(width  / 2);
		nOffsetY = (short)(height / 2);
	}
	public ZTFrame(short width, short height, short nXOff, short nYOff)
	{
		nWidth   = width;
		nHeight  = height;
		nOffsetX = nXOff;
		nOffsetY = nYOff;

		for (int i = 0; i < height; i++)
			vecBitmap.add(new Line());
	}

	public boolean loadFromFile(ByteBuffer bb)
		throws IOException
	{
		//
		// Start out clean
		//
		nWidth = nHeight = 0;
		nOffsetX = nOffsetY = 0;

		vecBitmap.clear();

		try
		{
			//
			// Overall size of frame bits
			//
			int nFrameSize = bb.getInt();

			//
			// Remember our current position
			//
			int nStartOfPixelBlock = bb.position();

			//
			// Height and width of bitmap, X and Y offsets
			//
			nHeight = bb.getShort();
			nWidth = bb.getShort();
			nOffsetY = bb.getShort();
			nOffsetX = bb.getShort();

			//
			// No idea what this value means...
			//
			short nUnused = bb.getShort();

			//
			// Read each line of the frame
			//
			for (int nLine = 0; nLine < nHeight; nLine++)
			{
				Line l = new Line();
				vecBitmap.add(l);

				//
				// Get the number of pixel sets in this line
				//
				byte yNumSets = bb.get();

				//
				// Read in and interpret each pixel set
				//
				for (byte ySet = 0; ySet < yNumSets; ySet++)
				{
					PixelSet ps = new PixelSet();
					l.vecPixelSets.add(ps);

					//
					// Number of transparent pixels in this set
					//
					ps.yNumTrans = (bb.get() & 0xFF);

					//
					// Number of color pixels in this set
					//
					int nNumColorPixels = (bb.get() & 0xFF);

					//
					// Add the color pixels to the set
					//
					for (int i = 0; i < nNumColorPixels; i++)
					{
						byte yPixel = bb.get();
						ps.vecColorPixels.add(new Short((short)(yPixel&0xFF)));
					}
				}
			}

			//
			// At this point we're done with this frame.
			// However some images have 9 or 10 bytes after the frame bits
			// which I have no idea what to do with...
			//
			// Anyway, let's just make sure we're at the beginning of
			// the next frame before we continue.
			//
			int nCurPos = bb.position();
			int nExpectedPos = nStartOfPixelBlock + nFrameSize;

			if (nCurPos < nExpectedPos)
				bb.position(nExpectedPos);
		}
		catch (BufferUnderflowException ex)
		{
			Object args[] = { new Integer(bb.position()) };
			throw new IOException(Res.format("ERR_ENDOFFRAME", args));
		}

		return true;
	}

	public boolean saveToFile(ByteBuffer bb)
		throws IOException
	{
		if (bb == null)
			return false;

		int nBufSize = 0;

		//
		// Calculate overall size of frame bits
		//
		Iterator itLines = vecBitmap.iterator();
		while (itLines.hasNext())
		{
			nBufSize++; // # of PixelSets in this line
			Line l = (Line)itLines.next();
			Iterator itPSets = l.vecPixelSets.iterator();
			while (itPSets.hasNext())
			{
				PixelSet ps = (PixelSet)itPSets.next();
				nBufSize += ps.vecColorPixels.size();
				nBufSize++; // # of transparent pixels in this set
				nBufSize++; // # of color pixels in this set
			}
		}
		nBufSize += 4; // 2 shorts: #lines, #cols
		nBufSize += 4; // 2 shorts: xoff, yoff
		nBufSize += 2; // 1 short: ??? always 0...

		//
		// Write out the data
		//
		try
		{
			bb.putInt(nBufSize);
			bb.putShort(nHeight);
			bb.putShort(nWidth);
			bb.putShort(nOffsetY);
			bb.putShort(nOffsetX);
			bb.putShort((short)0x00); // ???

			itLines = vecBitmap.iterator();
			while (itLines.hasNext())
			{
				Line l = (Line)itLines.next();
				byte yNumSets = (byte)l.vecPixelSets.size();
				bb.put(yNumSets);

				Iterator itPSets = l.vecPixelSets.iterator();
				while (itPSets.hasNext())
				{
					PixelSet ps = (PixelSet)itPSets.next();
					bb.put((byte)(ps.yNumTrans & 0xFF));

					byte yNumColorPixels = (byte)ps.vecColorPixels.size();
					bb.put(yNumColorPixels);

					Iterator itShort = ps.vecColorPixels.iterator();
					while (itShort.hasNext())
					{
						Short s = (Short)itShort.next();
						byte b = (byte)(s.shortValue() & 0xFF);
						bb.put(b);
					}
				}
			}
		}
		catch (BufferUnderflowException ex)
		{
			Object args[] = { new Integer(bb.position()) };
			throw new IOException(Res.format("ERR_ENDOFFRAME", args));
		}

		return true;
	}

	DisplayImage writeToImage(ZTPal ztpal)
//        BufferedImage writeToImage(Pal ztpal, int nImageType)
		throws ConversionException
	{
		//
		// Grab the transparent color now -- we're gonna use it a lot.
		// Force an alpha of 0 in case this is supported by the file format.
		//
		int rgbTrans = ZTPal.setA(ztpal.getTransparentColorARGB(), 0x00);

//		BufferedImage bi = new BufferedImage(nWidth, nHeight, nImageType);

		// Jay changed the following line to also put the offsets
		// in the created DisplayImage.

		DisplayImage bi =
		  new DisplayImage(nWidth, nHeight, getOffsets(),
		                   BufferedImage.TYPE_INT_ARGB);

		short nY = 0;
		Iterator itLines = vecBitmap.iterator();
		while (itLines.hasNext())
		{
			Line l = (Line)itLines.next();
			Vector vecBits = decodeLine(l);

			short nX = 0;
			Iterator itShort = vecBits.iterator();
			while (itShort.hasNext())
			{
				short s = ((Short)itShort.next()).shortValue();
				int rgb = 0;
				if (s == (short)-1)
				{
					//
					// Transparent pixel.
					//
					rgb = rgbTrans;
				}
				else
				{
					rgb = ztpal.getARGBColor(s);
				}
// TODO: Optimize this by saving one line at a time?
				bi.setRGB(nX, nY, rgb);
				nX++;
			}
			nY++;
		}
		return bi;
	}

	public Dimension getSize() { return new Dimension(nWidth, nHeight); }
	public void setSize(Dimension dim) { nWidth = (short)dim.getWidth(); nHeight = (short)dim.getHeight(); }
	public Point getOffsets() { return new Point(nOffsetX, nOffsetY); }
	public void setOffsets(Point point) { nOffsetX = (short)point.x; nOffsetY = (short)point.y; }

	/**
	 * Sets an entire line of the image based on the values contained
	 * in the array. We require nColorValues.length == nWidth.
	 *
	 * @param y The line number.
	 * @param nColorValues The array of pixel values for this line.
	 * @param ztpal The palette to use.
	 * @return true if the operation was successful.
	 * @throws ConversionException
	 */
	public boolean setPixelLine(short y, int nColorValues[], ZTPal ztpal)
		throws ConversionException
	{
		if (y >= nHeight)
		{
			Object args[] = { new Integer(y), new Integer(nHeight) };
			throw new ConversionException(Res.format("ERR_LINEOUTOFRANGE", args));
		}

		if (nColorValues.length > nWidth)
		{
			Object args[] = { new Integer(nColorValues.length), new Integer(nWidth) };
			throw new ConversionException(Res.format("ERR_LINETOOWIDE", args));
		}

		Vector vBits = new Vector(nWidth);

		for (int i = 0; i < nWidth; i++)
		{
			int color = nColorValues[i];
			short index = (short)ztpal.addARGBColor(color);
			vBits.add(new Short(index));
		}

		vecBitmap.set(y, encodeLine(vBits));

		return true;
	}

	public boolean setPixel(short x, short y, short cColorIndex)
		throws ConversionException
	{
		if (x >= nWidth || y >= nHeight)
		{
			Object args[] = { new Integer(x), new Integer(y), new Integer(nWidth), new Integer(nHeight) };
			throw new ConversionException(Res.format("ERR_PIXOUTOFRANGE", args));
		}

		if (y >= vecBitmap.size())
		{
			Object args[] = { new Integer(x), new Integer(y), new Integer(nWidth), new Integer(nHeight) };
			throw new ConversionException(Res.format("ERR_YVALOUTOFRANGE", args));
		}

		Vector vBits = decodeLine((Line)vecBitmap.get(y));

		if (vBits.size() < nWidth)
		{
			//
			// Fill up the line with transparent pixels
			//
			for (int i = vBits.size(); i < nWidth; i++)
				vBits.add(new Short((short)-1));
		}

		vBits.set(x, new Short(cColorIndex));

		vecBitmap.set(y, encodeLine(vBits));

		return true;
	}

	public Vector decodeLine(Line l)
	{
		Vector vecBits = new Vector();

		//
		// Go through all the PixelSets for this line
		//
		Iterator it = l.vecPixelSets.iterator();

		while (it.hasNext())
		{
			PixelSet ps = (PixelSet)it.next();
			//
			// Leading trans pixels
			//
			for (int i = 0; i < ps.yNumTrans; i++)
				vecBits.add(new Short((short)-1));

			//
			// Color pixels
			//
			Iterator itPix = ps.vecColorPixels.iterator();
			while (itPix.hasNext())
			{
				Short b = (Short)itPix.next();
				vecBits.add(b);
			}
		}
		return vecBits;
	}

	public Line encodeLine(Vector bits)
	{
		Line line = new Line();
		PixelSet ps = new PixelSet();

		line.vecPixelSets.add(ps);

		//
		// True if we're currently inside a transparent pixel block
		//
		boolean bInBlock = true;

		Iterator itBits = bits.iterator();
		while (itBits.hasNext())
		{
			Short b = (Short)itBits.next();
			if (b.intValue() == -1)
			{
				if (!bInBlock)
				{
					//
					// Start a new pixel set
					//
					ps = new PixelSet();
					line.vecPixelSets.add(ps);
					bInBlock = true;
				}
				ps.yNumTrans++;

				if (ps.yNumTrans == 255)
				{
					//
					// PixelSets are limited to 255 contiguous transparent pixels,
					// so start a new PixelSet here
					//
					bInBlock = false;
				}
			}
			else
			{
				bInBlock = false;
				ps.vecColorPixels.add(b);

				if (ps.vecColorPixels.size() == 255)
				{
					//
					// PixelSets are limited to 255 contiguous color pixels,
					// so start a new PixelSet here
					//
					ps = new PixelSet();
					line.vecPixelSets.add(ps);
					bInBlock = true;
				}
			}
		}

		return line;
	}
}

/********************************************************************
 *
 * $Log: /Zoot/src/com/madsci/zoot/ZTFrame.java $
 *
 * 6    11/11/09 3:32p Jay
 * Keep track of individual frame offsets.
 *
 * 5     7/13/03 12:14p Fpoulin
 * Fixed a bug in frame traversal logic.
 *
 * 4     7/11/03 1:42a Fpoulin
 * Optimized imports
 *
 * 3     7/08/03 4:50a Fpoulin
 * Performance improvements.
 *
********************************************************************/
