/////////////////////////////////////////////////////////////////////
//
// ZTTree.java :
//
//  The file tree view control.
//
/////////////////////////////////////////////////////////////////////
//
// $Revision: 7 $
// $Date: 7/17/03 12:53p $
//
/////////////////////////////////////////////////////////////////////

package com.madsci.zoot;

import java.awt.*;
import java.io.File;
import javax.swing.*;
import javax.swing.tree.*;

/**
 * The file tree view control.
 */
class ZTTree extends JTree
{
	private ZTFSTreeModel tm = null;

	public ZTTree(File f)
	{
		super (new ZTFSTreeModel(f));

		tm = (ZTFSTreeModel)getModel();

		getSelectionModel().setSelectionMode(
				TreeSelectionModel.SINGLE_TREE_SELECTION);

		DefaultTreeCellRenderer renderer = new ZTTreeCellRenderer();
		setCellRenderer(renderer);

		//tm.addTreeModelListener(new ZTTreeModelListener());
	}

	void refresh()
	{
		tm.refresh(new File(tm.getRootPath()));
	}

	void refresh(File f)
	{
		((ZTFSTreeModel)getModel()).refresh(f);
	}

	class ZTTreeCellRenderer extends DefaultTreeCellRenderer
	{
		Icon iconLeaf = new ImageIcon(com.madsci.zoot.MainFrame.class.getResource(
			"images/pic16x16.png"));
		Icon iconOpen = new ImageIcon(com.madsci.zoot.MainFrame.class.getResource(
			"images/openFolder16x16G.png"));
		Icon iconClosed = new ImageIcon(com.madsci.zoot.MainFrame.class.getResource(
			"images/closedFolder16x16G.png"));
		Icon iconSound = new ImageIcon(com.madsci.zoot.MainFrame.class.getResource(
			"images/sound.png"));
		Icon iconPal = new ImageIcon(com.madsci.zoot.MainFrame.class.getResource(
			"images/palette16x16.png"));
		Icon iconAnim = new ImageIcon(com.madsci.zoot.MainFrame.class.getResource(
			"images/animation16x16.png"));
		Icon iconText = new ImageIcon(com.madsci.zoot.MainFrame.class.getResource(
			"images/textDoc16x16.png"));

	    public Component getTreeCellRendererComponent(
	                        JTree tree,
	                        Object value,
	                        boolean sel,
	                        boolean expanded,
	                        boolean leaf,
	                        int row,
	                        boolean hasFocus)
	    {
	        super.getTreeCellRendererComponent(
	                        tree, value, sel,
	                        expanded, leaf, row,
	                        hasFocus);

		    if (!(tree instanceof ZTTree) || !(value instanceof ZTFSTreeModel.PseudoFile))
		    {
			    // Yikes
			    return this;
		    }

		    ZTTree zttree = (ZTTree)tree;
		    ZTFSTreeModel.PseudoFile ps = (ZTFSTreeModel.PseudoFile)value;

		    if (leaf)
		    {
			    if (ps.toString().lastIndexOf('.') == -1)
			    {
				    //
				    // If the selected file doesn't have an extension,
				    // let's try to load it into a ZTImage. If we succeed,
				    // display it in the main window.
				    //
					ZTFSTreeModel tm = (ZTFSTreeModel)zttree.getModel();

					ZTImage zti = new ZTImage(tm.getRootPath(), tm.getRelativePath(ps), 0);
					if (zti.isValidImage())
						setIcon(iconLeaf);
			    }
			    else if (ps.toString().lastIndexOf(".pal") != -1)
				    setIcon(iconPal);
			    else if (ps.toString().lastIndexOf(".wav") != -1)
				    setIcon(iconSound);
			    else if (ZootUtils.isTextFile(ps.toString()))
				    setIcon(iconText);
// TODO: Use custom icon for non-descript leaves
		    }
		    else
		    {
			    setIcon(expanded ? iconOpen : iconClosed);
		    }

	        return this;
	    }
	}
}

/********************************************************************
 *
 * $Log: /Zoot/src/com/madsci/zoot/ZTTree.java $
 * 
 * 7     7/17/03 12:53p Fpoulin
 * Added a ZTImage.isValidImage() call to boost tree expansion
 * performance.
 *
 * 6     7/13/03 12:13p Fpoulin
 * Added ability to preview text files.
 *
 * 5     7/11/03 1:42a Fpoulin
 * Optimized imports
 *
 * 4     7/08/03 4:41a Fpoulin
 * Added header and log comments.
 *
********************************************************************/
